
#ifndef PDL_ABSTRACTDRAWER
#define PDL_ABSTRACTDRAWER

#include "Model.h"
#include "RULES_Vector.h"

#include "ModelDrawer.h"

/**
*	Classe Abstraite d'affichage d'une classe Model en toonshading  l'cran en fonction
* de la position de la lumire.
*/
class AbstractDrawer : public ModelDrawer
{
protected :
	/**
	* la position de la lumire dans le repre de ce model
	*/
	RULES_Vector* lightPos;

	/**
	* l'identifiant OpenGL de la texture diffuse du model
	*/
	unsigned int texSkinID;

	/**
	* l'identifiant OpenGL de la texture Toon
	*/
	unsigned int texToonDiffuseID;

public:

	/**
	* Permet d'accder  la position de la lumire
	*
	* @return un pointeur sur la position de la lumire
	*/
	RULES_Vector* getLightPos();

	/**
	* Permet d'affecter la position de la lumire
	*
	* REM : pour que l'clairage soit correcte, la lumire doit tre exprimer
	* dans le repere du model.
	*
	* @param l pointeur sur la position de la lumire
	*/
	void setLightPos(RULES_Vector *l);

	/**
	* Permet d'affecter l'identifiant OpenGL de la texture diffuse
	*
	* @param id l'identifiant OpenGL de la texture diffuse
	*/
	void setTexSkinID(unsigned int id);

	/**
	* Permet d'accder  l'identifiant OpenGL de la texture diffuse
	*
	* @return l'identifiant OpenGL de la texture diffuse
	*/
	unsigned int getTexSkinID();

	/**
	* Permet d'affecter l'identifiant OpenGL de la texture Toon
	*
	* @param id l'identifiant OpenGL de la texture Toon
	*/
	void setTexToonDiffuseID(unsigned int id);

	/**
	* Permet d'accder  l'identifiant OpenGL de la texture Toon
	*
	* @return l'identifiant OpenGL de la texture Toon
	*/
	unsigned int getTexToonDiffuseID();

	/**
	* Dessine un model en toon shading avec sa texture diffuse.
	*
	* @param model le pointeur sur le model  afficher
	* @param modelPos le vecteur donnant la position de la lumire
	*/
	virtual void draw( Model *model, RULES_Vector modelPos);

	/**
	* Dessine un model en toon shading sans sa texture diffuse.
	*
	* @param model le pointeur sur le model  afficher
	* @param modelPos le vecteur donnant la position de la lumire
	* @param silhouette true si on doit dessinn la silhouette
	*/
	virtual void drawToonOnly( Model *model, RULES_Vector modelPos,bool silhouette);
};

#endif