// Camera.cpp: implementation of the Camera class.
//
//////////////////////////////////////////////////////////////////////

#include "Camera.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

Camera::Camera()
{
	// on initialise toutes les touches a faux
	for (int i=0;i<256;i++)
	{
		key[i]=false;
	}
	// on definit les touches de mouvement pas defaut
	avance='z';
	recule='s';
	strafeGauche='q';
	strafeDroite='d';
	tourneGauche='a';
	tourneDroite='e';
	saut=' ';
	accroupi='f';
	zoomIn='+';
	zoomOut='-';
	// niveau de zoom par defaut
	zoomLevel=70;
	// vitessed de la camera par defaut
	vitesse=0.25;

	// sensibilit de la souris par defaut
	mouseSensibilityY=0.0018;
	mouseSensibilityX=0.0012;
	// angle de la camera par defaut
	angleY=0;
	angleX=0;
	// position de la camera par defaut
	pos.setCoord(0,50,0);
	// direction par defaut
	direction.setCoord(0,0,1);
	// et axe par defaut
	axe.setCoord(0,1,0);
}

Camera::~Camera()
{
	// pas d'alloction dynamique => pas de supression
}

float Camera::getAngleX() {
	return angleX;
}

float Camera::getAngleY() {
	return angleY;
}

RULES_Vector Camera::getAxe() {
	return axe;
}

RULES_Vector Camera::getDirection() {
	return direction;
}

float Camera::getMouseSensibilityX() {
	return mouseSensibilityX;
}

float Camera::getMouseSensibilityY() {
	return mouseSensibilityY;
}

RULES_Vector Camera::getPos() {
	return pos;
}

float Camera::getVitesse() {
	return vitesse;
}

void Camera::setAngleX(float f) {
	angleX = f;
	processRotat();
}

void Camera::setAngleY(float f) {
	angleY = f;
	processRotat();
}

void Camera::addAngleX(float f) {
	angleX += f;
	processRotat();
}

void Camera::addAngleY(float f) {
	angleY += f;
	processRotat();
}

void Camera::setAxe(RULES_Vector vect) {
	axe = vect;
}

void Camera::setDirection(RULES_Vector vect) {
	direction = vect;
}

void Camera::setMouseSensibilityX(float f) {
	mouseSensibilityX = f;
}

void Camera::setMouseSensibilityY(float f) {
	mouseSensibilityY = f;
}

void Camera::setPos(RULES_Vector vect) {
	pos = vect;
}

void Camera::setVitesse(float f) {
	vitesse = f;
}


void Camera::keyPressed(int touche)
{
	key[touche]=true;
}

void Camera::keyReleased(int touche)
{
	key[touche]=false;
}

void Camera::processMove(int elapsedTime)
{
	// touch pour avancer
	if (key[avance])
	{
		RULES_Vector tmp = direction;
		// on bloque les mouvement de la camera sur l'axe Y (pour ne pas decoller du sol)
		tmp.setOneCoord(1,0);
		tmp.normalisation();
		tmp = tmp * (this->vitesse * elapsedTime);
		pos = pos + tmp;
	}
	// pour reculer
	if (key[recule])
	{
		RULES_Vector tmp = direction;
		// on bloque les mouvement de la camera sur l'axe Y (pour ne pas decoller du sol)
		tmp.setOneCoord(1,0);
		tmp.normalisation();
		tmp = tmp * (this->vitesse * elapsedTime);
		pos = pos - tmp;
	}
	// etc
	if (key[strafeGauche])
	{
		RULES_Vector tmp;
		direction.prodVectoriel(&axe,&tmp);
		tmp.normalisation();
		tmp = tmp * (this->vitesse * elapsedTime);
		// quand on strafe on avance moins vite (et en plus la on se deplace vers la gauche 
		// donc on ajoute un - :)
		tmp * (vitesse*0.75);
		pos = pos - tmp;

	}
	if (key[strafeDroite])
	{
		RULES_Vector tmp;
		axe.prodVectoriel(&direction,&tmp);
		tmp.normalisation();
		tmp = tmp * (this->vitesse * elapsedTime);
		// quand on strafe on avance moins vite (et en plus la on se deplace vers la gauche 
		// donc on ajoute un - :)
		tmp * (vitesse*0.75);
		pos = pos - tmp;
	}
	if (key[tourneGauche])
	{
	}
	if (key[tourneDroite])
	{
	}
	if (key[saut])
	{
	}
	if (key[accroupi])
	{
	}
	// code du zoom laissau cas ou on en aurrait besoin pour debugger l'affichage
	//TODO : borner le zoom pour vit dbordement
/*	if (key[zoomIn])
	{
		if(zoomLevel>20)
			zoomLevel-=0.1*elapsedTime;
		
		// en fait pour zoomer on reduit l'angle de FOV (appel ici zoomLevel)
		glMatrixMode(GL_PROJECTION);
		glLoadIdentity();
		gluPerspective(zoomLevel,512/384,1,10000);
		glMatrixMode(GL_MODELVIEW);
	}
	if (key[zoomOut])
	{
		if(zoomLevel<90)
			zoomLevel+=0.1*elapsedTime;
		// pour dezoomer on augmente l'angle de FOV
		glMatrixMode(GL_PROJECTION);
		glLoadIdentity();
		gluPerspective(zoomLevel,512/384,1,10000);
		glMatrixMode(GL_MODELVIEW);
	}*/
}

void Camera::look()
{
	// place la camera openGL
	gluLookAt (pos.getX(),pos.getY(),pos.getZ(),pos.getX()+direction.getX(),pos.getY()+direction.getY(),pos.getZ()+direction.getZ(),axe.getX(),axe.getY(),axe.getZ()); 
}

void Camera::processRotat()
{
	// on effectue les rotation des vecteurs
	RULES_Vector vecteur;
	vecteur.setCoord(0,0,1);
	vecteur.rotate(angleX,1,0,0);
	vecteur.rotate(angleY,0,1,0);
	direction.setOneCoord(0,vecteur.getX());
	direction.setOneCoord(1,vecteur.getY());
	direction.setOneCoord(2,vecteur.getZ());
}


bool Camera::isCurrentlyMoving()
{
	return (key[avance] | key[recule] | key[strafeGauche] | key[strafeDroite]);
}



float Camera::getZoomLevel()
{
	return (float)this->zoomLevel;
}
