// Camera.h: interface for the Camera class.
//
//////////////////////////////////////////////////////////////////////



#if !defined(CAMERA)
#define CAMERA

#ifdef WIN32
#include <windows.h>
#endif
#include <GL/GL.h>
#include "GL/glut.h"
#include <gl/glaux.h>
#include "RULES_Vector.h"	// Added by ClassView

/**
 * La classe de camera utilise par le jeu.
 *
 * Cette classe est une classe de camera basique. elle gere les mouvements de la camera en fonction des touches appuy
 * et la direction en fonction de la position de la souris.
 * la methode utilise n'est pas des plus intuitives. Il serait bon de la revoir :
 * - gerer la souris en interne de la camera et non pas a l'exterieur comme c'est fait a l'heure actuelle
 * - ajouter des methodes avancer, reculer, straf, monter, descendre... pour une interface publique
 * plus facile a utiliser. actuellement, on est oblig d'utiliser les numero de touche pour specifier le mouvement voulu...
 * - revoir la methode de gestion des mouvements, a l'heure actuelle, lorsqu'on appuie sur la touche pour avancer et pour
 * straffer, la camera se deplace plus vite (le vecteur de deplacement ne peut en faite pas etre normalise avec la
 * methode utilise...). Il faudrait cre un vecteur membre deplacement, auquel serait ajout les deplacement, puis ajouter
 * une methode qui effectue le mouvement (en normalisant deplacement...) cette methode permettrait en plus une utilisation
 * de la camera dans un moteur de collision plus evolu que celui de PDL...
 */
class Camera  
{
private:
	/**
	 * la vitesse de deplacement
	 */
	float vitesse;
	/**
	 * la sensibilite y de la souris
	 */
	float mouseSensibilityY;
	/**
	 * la sensibilite x de la souris
	 */
	float mouseSensibilityX;
	/**
	 * l'angle de vision Y
	 */
	float angleY;
	/**
	 * l'angle de vision X 
	 */
	float angleX;
	/**
	 * le FOV
	 */
	float zoomLevel;
	/**
	 * la direction ou regarde la camera
	 */
	RULES_Vector direction;
	/**
	 * l'axe de regard (pour gluLookAt)
	 */
	RULES_Vector axe;
	/**
	 * le temps au moment de la derniere action
	 */
	float lastTime;
	/**
	 * l'ensemble des touches de la table ascii
	 */
	bool key[256];
	/**
	 * mthode interne de gestion des rotations
	 */
	void processRotat();
protected:
	/**
	 * l position de la camera
	 */
	RULES_Vector pos;
public:

	/**
	 * methode permettant de connaitre le niveau de zoom de la camera
	 * @return float: le niveau de zoom
	 */
	float getZoomLevel();
	
	/**
	 * le numero de la touche utiliser pour faire avancer la camera
	 */
	int avance;

	/**
	 * le numero de touche utilis pour faire reculer la camera
	 */
	int recule;

	/**
	 * le numero de touche utilis pour straffer a gauche
	 */
	int strafeGauche;

	/**
	 * le numero de touche uilis pour straffer a droite
	 */
	int strafeDroite;

	/**
	 * le numero de touche utilise pour tourner a gauche
	 */
	int tourneGauche;

	/**
	 * le numero de touche utilise pour tourner a droite
	 */
	int tourneDroite;

	/**
	 * le numero de touche utilise pour sauter
	 */
	int saut;

	/**
	 * le numero de touche utilise pour s'accroupire
	 */
	int accroupi;

	/**
	 * le numero de touche utilise pour zoomer
	 */
	int zoomIn;

	/**
	 * le numero de touche utilise pour dezoomer
	 */int zoomOut;
	
	/**
	 * constructeur par defaut, initialise la position de la camera a 0,50,0, l'angle de vue et les touches par defaut.
	 */
	Camera();

	/**
	 * destructeur de la camera
	 */
	virtual ~Camera();

	/**
	 * retourne l'angle de rotation de la camera sur l'axe X
	 * @return float: l'angle de la camera sur l'angle X
	 */
	float getAngleX();

	/**
	 * retourne l'angle de rotation de la camera sur l'axe Y
	 * @return float: l'angle de la camera sur l'angle Y
	 */
	float getAngleY();

	/**
	 * retourne l'axe d'orientation de la camera. cet axe definit le haut de la camera (c'est le 3me vecteur pass a 
	 * gluLookAt())
	 * @return RULES_Vector: vecteur representant l'axe
	 */
	RULES_Vector getAxe();

	/**
	 * retourne le vecteur de la direction de la camera.
	 * @return RULES_Vector: la direction de la camera
	 */
	RULES_Vector getDirection();

	/**
	 * retourne la sensibilit de la souris sur l'axe X
	 * @return float: la sensibilite de la souris sur l'axe X
	 */
	float getMouseSensibilityX();

	/**
	 * retourne la sensibilit de la souris sur l'axe Y
	 * @return float: la sensibilite de la souris sur l'axe Y
	 */
	float getMouseSensibilityY();

	/**
	 * retourne la position de la camera dans l'espace global
	 * @return RULES_Vector: la position de la camera
	 */
	RULES_Vector getPos();

	/**
	 * retourne la vitesse de deplacement de la camera.
	 * @return float: la vitesse de deplacement de la camera
	 */
	float getVitesse();

	/**
	 * definit l'angle de la camera sur l'axe X
	 * @param l'angle de la camera sur l'axe X
	 */
	void setAngleX(float);

	/**
	 * definit l'angle de la camera sur l'axe Y
	 * @param l'angle de la camera sur l'axe Y
	 */
	void setAngleY(float);

	/**
	 * ajoute un angle a l'angle de la camera sur l'axe X
	 * @param l'angle a ajouter a celui de la camera
	 */
	void addAngleX(float);

	/**
	 * ajoute un angle a l'angle de la camera sur l'axe Y
	 * @param l'angle a ajouter a celui de la camera
	 */
	void addAngleY(float);

	/**
	 * definit l'axe de la camera
	 * @param L'axe de la camera
	 */
	void setAxe(RULES_Vector);

	/**
	 * definit la direction dans laquelle regarde la camera
	 * @param la direction de la camera
	 */
	void setDirection(RULES_Vector);

	/**
	 * definit la sensibilit de la camera sur l'axe X
	 * @param la sensibilite de la camera
	 */
	void setMouseSensibilityX(float);

	/**
	 * definit la sensibilit de la camera sur l'axe Y
	 * @param la sensibilite de la camera
	 */
	void setMouseSensibilityY(float);

	/**
	 * definit la position de la camera
	 * @param la position de la camera
	 */
	void setPos(RULES_Vector);

	/**
	 * definit la vitesse de la camera
	 * @param la vitesse de la camera
	 */
	void setVitesse(float);

	/**
	 * signale a la camera qu'une touche a t appuy
	 * @param le code GLUT de la touche appuy
	 */
	void keyPressed(int);

	/**
	 * signale a la camera qu'une touche a t relache
	 * @param le code GLUT de la touche appuy
	 */
	void keyReleased(int);

	/**
	 * effectue le deplacement de la camera
	 * @param elapsedTime: le temps ecoul depuis le dernier mouvement de camera
	 */
	void processMove(int elapsedTime);

	/**
	 * envoie a openGL la position de la camera
	 */
	void look();

	/**
	 * permet de savoir si la camera est actuellement en mouvement
	 * @return bool:
	 * - true: la camera est en mouvement
	 * - false: la camera ne bouge pas
	 */
	bool isCurrentlyMoving();

};

#endif
