
#ifndef PDL_ENHANCEDRENDER
#define PDL_ENHANCEDRENDER

#include "AbstractDrawer.h"
#include "arb_multitexture.h"

#ifdef WIN32
#include <windows.h>
#endif
#include <GL\gl.h>
#include "GL\glut.h"

/**
*		Classe d'affichage d'une classe Model en toonshading  l'cran. Le rendu est pour
*	tout les cartes supportant au moins 2 unit de multitexturingARB.
*		==> 1 pass.
*		==> Calcul toonshading avec le CPU.
*
*		La carte doit supporter :
*			- les objet de textures (OpenGL 1.2)
*			- les tableau de coordonne (OpenGL 1.1)
*			- au minimum 2 units de textures pour le multitexturingARB.
*/
class EnhancedRender : public AbstractDrawer
{
private:
	/**
	*	Le dernier nombre de coordonne de texture allouer
	* Permet d'viter de redfinir pour chaque dessin le tableau des coord de
	* texture utilis pour le rendu toon allou en mmoire system.
	*/
	unsigned int lastNumberOfToonTexCoord;
	
	/**
	*	Le tableau de coordonne de texture Toon.
	*/
	float *diffuseToonTexCoord;

public:

	/**
	* Dessine un model en toon shading avec sa texture diffuse.
	*
	*	prcondition :
	* - aucune option OpenGL pour l'affichage n'a t activ (blending, depth, vertexprogram....)
	*
	*
	* @param model le pointeur sur le model  afficher
	* @param modelPos le vecteur donnant la position de la lumire
	*/
	void draw(Model *model,RULES_Vector modelPos);

	/**
	* Dessine un model en toon shading sans sa texture diffuse.
	*
	*	prcondition :
	* - aucune option OpenGL pour l'affichage n'a t activ (blending, depth, vertexprogram....)
	*
	* @param model le pointeur sur le model  afficher
	* @param modelPos le vecteur donnant la position de la lumire
	* @param silhouette true si on doit dessinn la silhouette
	*/
	void drawToonOnly( Model *model, RULES_Vector modelPos,bool silhouette);
	
	/**
	* Constructeur
	*/
	EnhancedRender();

	/**
	* Destructeur
	*/
	~EnhancedRender();
};


#endif