
#ifndef PDL_FASTRENDER
#define PDL_FASTRENDER

#include "AbstractDrawer.h"
#include "arb_multitexture.h"

#ifdef WIN32
#include <windows.h>
#endif
#include <GL\gl.h>
#include "GL\glut.h"


/**
* Le vertex_program ARB du rendu toon
*/
const unsigned char VP_TOON_ARB[] = 
	"!!ARBvp1.0"
	"#\n"
	"# Vertex_Program_ARB pour toon-shading de POUTRAGE DE LOUTRE\n"
	"# par Hillaire Sebastien\n"
	"#\n"

	"# affecte les alias des entres, sorties, variables locales...\n"
	"# l'attribut de vertex 69 contient la coordone de la lumire orient par rapport au model\n"
	"ATTRIB iPos	= vertex.position;"
	"ATTRIB iNrml	= vertex.normal;"
	"ATTRIB iTex0	= vertex.texcoord[0];"
	"ATTRIB iTex1	= vertex.texcoord[1];"
	"ATTRIB iCol	= vertex.color;"
	"ATTRIB iFog	= vertex.fogcoord;"
	"PARAM mvp[4]	= { state.matrix.mvp };"
	"PARAM lightpos	= program.env[69];"
	"TEMP tmp0;"
	"OUTPUT oPos	= result.position;"
	"OUTPUT oTex0	= result.texcoord[0];"
	"OUTPUT oTex1	= result.texcoord[1];"
	"OUTPUT oFog	= result.fogcoord;"
	"OUTPUT oCol	= result.color;"

	"# calcul des coordonne de la texture toon\n"
	"SUB tmp0, lightpos, iPos;"
	"DP3 tmp0.w, tmp0, tmp0;"
	"RSQ tmp0.w, tmp0.w;"
	"MUL tmp0.xyz, tmp0, tmp0.w;"
	"DP3 oTex0.x, iNrml, tmp0;"
	"MOV oTex0.y, {0};"
	"MOV oTex1, iTex0;"

	//TODO faire un 2me vp pour le toon only
	"MOV oCol, iCol;"

	"# transforme le vertex dans l'espace de coordone homogne\n"
	"DP4 oPos.x, mvp[0], iPos;"
	"DP4 oPos.y, mvp[1], iPos;"
	"DP4 oPos.z, mvp[2], iPos;"
	"DP4 oPos.w, mvp[3], iPos;"
	"END";


//TODO essayer d'optimiser en utilisant seulement R0 (mais il est possible que cela n'apporte rien)
//TODO voir si v[NMRL] est bien l'idntifiant de la normal
/**
* Le vertex_program NV du rendu toon
* REM : non utlis et non test...
*/
const unsigned char VP_TOON_NV[] = 
	"!!VP1.1\n"
	"#\n"
	"# Vertex_Program_ARB pour toon-shading de POUTRAGE DE LOUTRE\n"
	"# par Hillaire Sebastien\n"
	"#\n"

	"# l'attribut de vertex 69 contient la coordone de la lumire orient par rapport au model\n"
	"SUB R0, c[69], v[OPOS];"
	"DP3 R1.w, R0, R0;"
	"RSQ R1.w, R1.w;"
	"MUL R0.xyz, R0, R1.w;"
	"DP3 o[TEX0].x, v[NMRL], R0;"
	"MOV o[TEX1], v[TEX1];"

	"DP4 o[HPOS].x, v[OPOS], c[0];"
	"DP4 o[HPOS].y, v[OPOS], c[1];"
	"DP4 o[HPOS].z, v[OPOS], c[2];"
	"DP4 o[HPOS].w, v[OPOS], c[3];"

	"END;";


/**
*	Classe d'affichage d'une classe Model en toonshading  l'cran. Le rendu est pour
*	tout les cartes supportant au moins 2 unit de multitexturingARB et les vertexProgramARB.
*		==> 1 pass.
*		==> Calcul toonshading avec le GPU.
*
*		La carte doit supporter :
*			- les objet de textures (OpenGL 1.2)
*			- les tableau de coordonne (OpenGL 1.1)
*			- au minimum 2 units de textures pour le multitexturingARB.
*			- les vertexprogram ARB ou NV
*
* Comment par Hillaire Sbastien
*/
class FastRender : public AbstractDrawer
{
private:
	/**
	* L'identifiant OpenGL du vertex_program
	*/
	unsigned int vpID;

public:
	/**
	* Constructeur
	*/
	FastRender();

	/**
	* Destructeur
	*/
	~FastRender();

	/**
	* Dessine un model en toon shading avec sa texture diffuse.
	* prcondition :
	*	- aucune option OpenGL pour l'affichage n'a t activ (blending, depth....)
	*
	* @param model le pointeur sur le model  afficher
	* @param modelPos le vecteur donnant la position de la lumire
	*/
	void draw(Model *model,RULES_Vector modelPos);

	/**
	* Dessine un model en toon shading sans sa texture diffuse.
	* prcondition :
	*	- aucune option OpenGL pour l'affichage n'a t activ (blending, depth....)
	*
	* @param model le pointeur sur le model  afficher
	* @param modelPos le vecteur donnant la position de la lumire
	* @param silhouette true si on doit dessinn la silhouette
	*/
	void drawToonOnly( Model *model, RULES_Vector modelPos,bool silhouette);

	/**
	* Permet d'assigner le verte_program de toon shading
	*
	* @param id L'identifiant OpenGL du vertex_program
	*/
	void setVPID(unsigned int *id);
};


#endif
