// Font.cpp: implementation of the Font class.
//
//////////////////////////////////////////////////////////////////////

#include "Font.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

Font::Font()
{
	// initialisation de base de la classe de font
	// en fait ca corespond a la configuration pour la texture de base
	int charWidth = 15;
	int charHeight = 15;
	int espace = 10; 
	int xCharNumber = 15;
	int yCharNumber = 15;
	int screenWidth = 640;
	int screenHeight = 480;
	int premierChar = 0;

}

Font::~Font()
{
	// on enleve les display list de la memoire
	glDeleteLists(font,xCharNumber * yCharNumber);		

}

void Font::buildFont()
{
	// on prend la largeur d'un caractere dans la texture, la texture allant de 0 a 1 c'est donc
	// 1/(taille d'un charactere)
	float Xscale=1.f/(xCharNumber+1);
	// idem avec la hauteur
	float Yscale=1.f/(yCharNumber+1);
	// une variable temporaire qui permettra de savoir le numero du caractere sur lequel on
	// travail
	int number=0;

	// ici on cre les display liste qui contiendront les caracteres. on en cre autant qu'il
	// y a de caracteres dans la texture, c'est a dire x*y caracteres
	font=glGenLists(xCharNumber*yCharNumber);
	
	// boucle qui lit les lignes. le probleme c'est qu'avec openGL les coordonne de texture
	// sont a (0,0) en bas a gauche et (1,1) en haut a droite or nous avons une texture
	// qu'il faut lire de bas en haut c'est pourquoi la boucle est a l'envers.
	for (int j=yCharNumber;j>0;j--)
	{
		// idem pour les colonnes mais sans inversion de la boucle qui est a l'endroit cette fois
		// ci car on veut effectivement lire de gauche a droite.
		for (int i=0;i<=xCharNumber;i++)
		{
			// on cre la liste du caractere sur lequel on travail
			glNewList(font+number,GL_COMPILE);
			{
				// on cre le carr qui contient le caractere (a assaier avec triangle_Strip)
				glBegin(GL_QUADS);
				{
					glTexCoord2f(i*Xscale,j*Yscale);glVertex2f(0,0);
					glTexCoord2f(Xscale*i+Xscale,j*Yscale);glVertex2f(charWidth,0);
					glTexCoord2f(Xscale*i+Xscale,j*Yscale+Yscale);glVertex2f(charWidth,charHeight);
					glTexCoord2f(i*Xscale,j*Yscale+Yscale);glVertex2f(0,charHeight);
				}
				glEnd();
				// et on translate de l'espace entre chaque caractere.
				glTranslated(espace,0,0);
				// et on passe a la display liste suivante
				number++;

			}
			glEndList();
			// fin de la liste

		}
	}
	// fin des boucles
	// fin de la methode


}

void Font::glText2d(int x, int y, const char *chaine, ...)
{
	// on cree une liste de 256 caractere
	char		texte[256];
	// on cre une va_list qui va recuperer les info pass en parametre par les "..."
	va_list		argsPtr;

	// et on traduit la chaine de caractere passe par "chaine"
	va_start(argsPtr, chaine);
	vsprintf(texte, chaine, argsPtr);
	va_end(argsPtr);
	// maintenant on a une chaine de caractere ou les %d,%f et autre ont ete remplac par
	// les valeur pass en parametre par les "...". en gros notre methode fonctionne comme
	// printf() !!


	// on selectionne le texture de la font
	glBindTexture(GL_TEXTURE_2D, texture_ID);
	glTexEnvf(GL_TEXTURE_ENV,GL_TEXTURE_ENV_MODE,GL_MODULATE);
	
	// on desactive le depth test
	glDisable(GL_DEPTH_TEST);
	
	// on passe en mode projection
	glMatrixMode(GL_PROJECTION);
	
	// on sauvegarde la matrice de projection
	glPushMatrix();
	
		// on nettoie la matrice de projection
		glLoadIdentity();
		
		// on passe en mode ortho de la taille de l'ecran
		glOrtho(0,screenWidth,0,screenHeight,-1,1);	
		
		// on passe en mode modelview
		glMatrixMode(GL_MODELVIEW);
		
		// on sauvegarde la matrice de modelview
		glPushMatrix();
		
			// on nettoie la matrice de projection
			glLoadIdentity();
			
			// on positionne le text
			glTranslated(x,screenHeight - y - charHeight,0);
			
			// on selectionne la premiere display liste (la font de base de notre liste de caractere)
			glListBase(font - premierChar);
			
			// on affiche le text
			glCallLists(strlen(texte),GL_UNSIGNED_BYTE,texte);
			// on selectionne la matrice de projection
			glMatrixMode(GL_PROJECTION);
		
		// et on recupere la matrice de projection sauvegard precedement
		glPopMatrix();	
		
		// on selectionne la matrice de projection modelview
		glMatrixMode(GL_MODELVIEW);
	
	// et on recupere la matrice de modelview sauvegard precedement
	glPopMatrix();
	
	// puis on reactive le depth test
	glEnable(GL_DEPTH_TEST);

}


void Font::setScreenSize(int width,int height)
{
	screenWidth=width;
	screenHeight=height;
}


void Font::setCaractereSize(int width,int height,int space)
{
	charHeight=height;
	charWidth=width;
	espace=space;
}


void Font::setTextureProperties(unsigned int tex_ID,int xNumber,int yNumber)
{
	texture_ID=tex_ID;
	xCharNumber=xNumber;
	yCharNumber=yNumber;
}


void Font::setBaseCaractere(int base)
{
	premierChar=base;
}


