// Frustum.h: interface for the Frustum class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(FRUSTUM_H)
#define FRUSTUM_H

#ifdef WIN32
#include <windows.h>
#endif
#include <GL\gl.h>
#include <GL\glut.h>/*D:\Programmation\C++\Librairie\Open*/
#include <gl\glext.h> 
#include <math.h>
#include "Objet.h"

/**
* Enumeration des position possible d'un objet par rapport au frustum
*/
enum Frustum_Pos
{
	/**
	* l'objet est compltement en dehors du frustum
	*/
	dehors = 0,
	/**
	* l'objet est en partie dans le frustum
	*/
	part_dedans,
	/**
	* l'objet est entirement dans frustum
	*/
	dedans
};

/**
* La classe permettant de grer le fustrum de vision et de savoir si un objet est dedans.
*Des outil pour les portal sont notement intgr.
*
* Comment par Hillaire Sbastien
*/
class Frustum  
{
private:
	/**
	* 6 plans
	*/
	float frustum[6][4];
	/**
	* ModelView sauvegarde
	*/
	float old_ModelView[16];
	/**
	* Projection sauvegarde
	*/
	float old_Projection[16];
	/**
	* ModelView
	*/
	float modl[16];
	/**
	* Projection
	*/
	float proj[16];
	/**
	* le frustum de vue
	*/
	float clip[16];
	/**
	* est ce que l une des 2 matrices a change ?
	* true si oui.
	*/
	bool change;

public:

	/**
	* Permet de savoir si un objet est dans le frustum grace  sa bounding box
	*
	* @param obj l'objet  tester
	*/
	bool ObjetInFrustum(Objet * obj, float size = 1.0f);

	/**
	* Constructeur
	*/
	Frustum();

	/**
	* Destructeur
	*/
	virtual ~Frustum();

	/**
	* Permet d'extraire la matrice
	*
	* @param extraire_mat true pour extraire la matrice
	*/
	void Extrait(bool extraire_mat);

	/**
	* Permet de vrifier si la matrice a changer pour recalculer ou non les plan de clipping
	*
	* @return true : la matrice est inchang
	*/
	bool SameMatrix();

	/**
	* Permet de spcifier qu'il faut tout recalculer
	*
	* @param b_changed true : il faut tout recalculer
	*/
	void SetChanged(bool b_changed);

	/**
	* Permet de spcifier un portal de clipping
	*
	* @param cameraX
	* @param cameraY
	* @param cameraZ
	* @param x1 coord x du premier point du portal
	* @param y1 coord y du premier point du portal
	* @param z1 coord z du premier point du portal
	* @param x2 coord x du deuxime point du portal
	* @param y2 coord y du deuxime point du portal
	* @param z2 coord z du deuxime point du portal
	* @param x3 coord x du troisime point du portal
	* @param y3 coord y du troisime point du portal
	* @param z3 coord z du troisime point du portal
	* @param x4 coord x du quatrime point du portal
	* @param y4 coord y du quatrime point du portal
	* @param z4 coord z du quatrime point du portal
	*/
	void SetPortal(float cameraX,float cameraY,float cameraZ,float x1,float y1,float z1,float x2,float y2,float z2,float x3,float y3,float z3,float x4,float y4,float z4);

	/**
	* Permet de spcifier un portal de clipping
	*
	* @param camera la position de la camra
	* @param portal les quatre point du portal
	*/
	void setPortalv(float camera[3],float portal[4][3]);


	/**
	* Vrifie si un point est dans le frustum
	*
	* @param x coord x du point a test
	* @param y coord y du point a test
	* @param z coord z du point a test
	* @return une numration de visibilit
	*/
	int PointInFrustum( float x, float y, float z );

	/**
	* Vrifie si une sphre est dans le frustum
	*
	* @param x coord x du centre de la sphere a test
	* @param y coord y du centre de la sphere a test
	* @param z coord z du centre de la sphere a test
	* @param radius le rayon de la sphere
	* @return une numration de visibilit
	*/
	int SphereInFrustum( float x, float y, float z, float radius );

	/**
	* Vrifie si un cube est dans le frustum
	*
	* @param x coord x du centre du cube a test
	* @param y coord y du centre du cube a test
	* @param z coord z du centre du cube a test
	* @param size la taille de la sphere par rapport au CENTRE.
	* @return une numration de visibilit
	*/
	int CubeInFrustum( float x, float y, float z, float size );

	/**
	* Vrifie si un rectangle est dans le frustum
	*
	* @param x1 coord x du premier point du rectangle
	* @param y1 coord y du premier point du rectangle
	* @param z1 coord z du premier point du rectangle
	* @param x2 coord x du deuxime point du rectangle
	* @param y2 coord y du deuxime point du rectangle
	* @param z2 coord z du deuxime point du rectangle
	* @param x3 coord x du troisime point du rectangle
	* @param y3 coord y du troisime point du rectangle
	* @param z3 coord z du troisime point du rectangle
	* @param x4 coord x du quatrime point du rectangle
	* @param y4 coord y du quatrime point du rectangle
	* @param z4 coord z du quatrime point du rectangle
	* @return une numration de visibilit
	*/
	int RectInFrustum( float x1,float y1,float z1,float x2,float y2,float z2,float x3,float y3,float z3,float x4,float y4,float z4);

	/**
	* Vrifie si un rectangle est dans le frustum
	*
	* @param rect les quatre point du rectangle
	* @return une numration de visibilit
	*/
	int RectInFrustumv(float rect[4][3]);

};

#endif 
