// Game.cpp: implementation of the Game class.
//
//////////////////////////////////////////////////////////////////////

#include "Game.h"


//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////
Game::Game()
{
	// a l'initialisation, le joueur ne tire pas
	hasJustFired = false;

	// difficulte par defaut
	this->difficulty = 11;
	
	// le souris est en mode normale par defaut
	mouseFactor = 1;

	// le rayon limite du terrain
	this->limiteRadius = 1000;

	// initialisation du warning
	warningValue[0] = 1;
	warningValue[1] = 0.75;
	warningValue[2] = 0.5;
	warningFactor[0] = 1;
	warningFactor[1] = 1;
	warningFactor[2] = 1;

	// le jeu n'est pas en pause (ca il n'est pas encore demarr)
	paused = false;
	
	// chargement des textures non dependantes du niveau de detail
	RULES_Texture * fire = new RULES_Texture();
	fire->loadBMP("./ressources/textures/fire.bmp",true,GL_LINEAR_MIPMAP_LINEAR,GL_LINEAR,GL_CLAMP,GL_CLAMP,0,true,255,0,0,0,200);
	RULES_Texture * crosshair = new RULES_Texture;
	crosshair->loadBMP("./ressources/textures/crosshair.bmp",0,GL_LINEAR_MIPMAP_LINEAR,GL_LINEAR,GL_CLAMP,GL_CLAMP,0,true,0,0,255,0,1);
	toon = new RULES_Texture();
	toon->loadBMP("./ressources/textures/toon.bmp",false,GL_LINEAR,GL_LINEAR,GL_CLAMP,GL_CLAMP,0,false,255,0,0,0,1);
	toon_glow = new RULES_Texture();
	toon_glow->loadBMP("./ressources/textures/toon_glow.bmp",false,GL_LINEAR,GL_LINEAR,GL_CLAMP,GL_CLAMP,0,false,255,0,0,0,1);
	lightMap = new RULES_Texture();
	
	//chargement texture HUD
	hud_rbalayeur=new RULES_Texture();
	hud_rbalayeur->loadBMP("./ressources/textures/HUD/balayeur.bmp",false,GL_LINEAR_MIPMAP_LINEAR,GL_LINEAR,GL_CLAMP,GL_CLAMP,0,false,255,0,0,0,200);
	hud_rpoint=new RULES_Texture();
	hud_rpoint->loadBMP("./ressources/textures/HUD/point.bmp",false,GL_LINEAR_MIPMAP_LINEAR,GL_LINEAR,GL_CLAMP,GL_CLAMP,0,true,0,0,0,0,255);
	hud_radar=new RULES_Texture();
	hud_radar->loadBMP("./ressources/textures/HUD/radar.bmp",false,GL_LINEAR_MIPMAP_LINEAR,GL_LINEAR,GL_CLAMP,GL_CLAMP,0,true,0,0,0,0,255);
	
	// ces 2 texture ne doivent pas etre supprim pas cette classe car elles seront associe 
	// a un model qui les detruira
	RULES_Texture * loutreTexture = new RULES_Texture();
	RULES_Texture * ImmeubleTexture = new RULES_Texture();
	
	// creation de la skybox
	skyBox = new SkyBox();
	skyBox->back = new RULES_Texture();
	skyBox->bottom = new RULES_Texture();
	skyBox->front = new RULES_Texture();
	skyBox->left = new RULES_Texture();
	skyBox->right = new RULES_Texture();
	skyBox->top = new RULES_Texture();
	
	
	// on charge les fonction de multitexturing et de vertex program
	getARBMultitextureProc();
	getARBVertexProgramProc();
	
	
	//SELECTION DU BON MODE DE RENDU
	if(ARB_Multitexture_IsDefined)
	{
		// on a le multitexturing
		if(ARB_Vertex_Program_IsDefined)
		{
			// et on a les vertex program
			// on utilise donc le code path fastRender
			FastRender * tempDrawer = new FastRender();
			tempDrawer->setTexToonDiffuseID(toon->getId());
			// le drawer est un fastRender
			this->drawer = tempDrawer;
		}
		else
		{
			// on utilise le enhancedRender
			EnhancedRender * tempDrawer = new EnhancedRender();
			tempDrawer->setTexToonDiffuseID(toon->getId());
			this->drawer = tempDrawer;
		}
	}
	else
	{
		// utilisation du basic render
		BasicRender * tempDrawer = new BasicRender();
		tempDrawer->setTexToonDiffuseID(toon->getId());
		this->drawer = tempDrawer;
	}
	
	// initialisation de la lumiere
	lightPos = new RULES_Vector();
	lightPos->setCoord(500,200,0);
	lightRadius = 1000;
	
	// initialisation du joueur
	joueur = new Joueur();
	joueur->setModelDrawer(this->drawer);
	joueur->getModel()->setTexture(new RULES_Texture());
	joueur->setFireTexture( fire);
	joueur->setShootFrequency(70);
	joueur->setDisplayFireTime(60);
	joueur->setCrossHairTex(crosshair);
	joueur->setScreenAttributs(1024,768);
	joueur->setLight(lightPos);
	
	// creation de la texture du sol (qui sera charg par la suite en fonction du niveau de detail)
	sol = new RULES_Texture;
	
	// on charge les models
	this->loutreModel = new Model();
	loutreModel->load("./ressources/models/loutre.obj");
	// par defaut le model de la loutre n'est pas la loutre low detail
	this->useLoutreLow = false;
	loutreModel->setTexture(loutreTexture);
	this->immeubleModel = new Model();
	immeubleModel->load("./ressources/models/immeuble.obj");
	immeubleModel->setTexture(ImmeubleTexture);
	
	// on crr l'immeuble
	Immeuble * immeuble = new Immeuble();
	immeuble->setModel(immeubleModel);
	immeuble->setModelDrawer(this->drawer);
	immeuble->setLight(lightPos);
	
	
	// on cre les loutres en fonction du niveau de difficulte
	Loutre * loutre = new Loutre[difficulty+10];
	for (int i = 0; i<difficulty+10; i++)
	{
		loutre[i].setIncrAngle(20);
		loutre[i].setCurrentAngle(0);
		loutre[i].setMaxAngle(5);
		loutre[i].setModelDrawer(this->drawer);
		loutre[i].setModel(loutreModel);
		loutre[i].setVie(10);
		loutre[i].setSpeed(0.01);
		loutre[i].setLight(lightPos);
		loutre[i].setDestination(immeuble->getPosition());
	}
	
	// on cre un loutre manager qui s'occupera de loutres
	loutreManager = new LoutreManager();
	loutreManager->setCreationRadius(2000);
	// on lui asigne les loutres
	loutreManager->setLoutres(loutre, difficulty+10);
	// et l'immeuble
	loutreManager->setImmeuble(immeuble);
	
	// creation du collision manager
	collisionManager = new CollisionManager(loutre, loutreManager->getImmeuble(), joueur, difficulty+10, 1, 1);
	
	// creation du quadrique glu qui sert de limite du terrain
	limite = gluNewQuadric(); 
	
	// creation de la font utilise pour afficher du text pendant le jeu
	font = new Font();
	texFont = new RULES_Texture();
	texFont->loadBMP("./ressources/textures/Font.bmp",true,GL_LINEAR_MIPMAP_LINEAR,GL_LINEAR,GL_REPEAT,GL_REPEAT,0,false,255,0,255,0,1);
	// initialisation de la font
	font->setTextureProperties(texFont->getId(),15,15);
	font->setBaseCaractere(32);
	font->setCaractereSize(15,15,10);
	// on met une fausse taille d'ecran pour avoir de text plus gros
	font->setScreenSize(800,600);
	font->buildFont();
	
	// creation de la classe qui se charge du motion blur
	motionBlur = new MotionBlur();
	
	// creation d'un frustum de vue
	frustum = new Frustum();
	
	//init pour le calcul du score
	startTimeForScore=0;
	nbKilledLoutreForScore=0;
	nbShoot=0;
	nbGoodShoot=0;
	
	//radar
	hud_balayeur_alpha=0;
	hud_balayeur_deltaAlpha=0.1;
	
	// initialisation du glow
	glow = new Glow(64,64);
	glow->setNbPass(2);
	glow->setGlowPower(1);
	glowLevel = 2;
	
	
	// on charge l'ensemble des textures dont le niveau de qualite depend du niveau de detail.
	loadRessources(0, 3, false);

}

void Game::startGame()
{
	// mise du score a 0
	joueur->setScore(0);
	
	// la limite du terrain est de 1000. A l'origine elle devait varier avec la difficulte
	// mais ca napportait rien...
	this->limiteRadius = 1000;
	
	// on supprime les loutres deja existantes (celles de la partie precedente)
	delete [] loutreManager->getLoutre(0);
	
	// on cre des nouvelles loutres toutes fraiches...
	Loutre * loutre = new Loutre[difficulty+10];
	for (int i = 0; i<difficulty+10; i++)
	{
		// et on les initialises.
		loutre[i].setIncrAngle(20);
		loutre[i].setCurrentAngle(0);
		loutre[i].setMaxAngle(5);
		loutre[i].setModelDrawer(this->drawer);
		loutre[i].setModel(loutreModel);
		loutre[i].setVie(1+(int)(difficulty/5));
		loutre[i].setSpeed((0.03 +0.03* (int)(difficulty/10)));
		loutre[i].setLight(lightPos);
		loutre[i].setDestination(loutreManager->getImmeuble()->getPosition());
	}
	
	// on configure le manager de loutres
	loutreManager->setCreationRadius(this->limiteRadius + 500 );
	loutreManager->setLoutres(loutre, difficulty+10);
	loutreManager->getImmeuble()->setVie(50 - difficulty);
	
	// on configure le manager de collisions
	collisionManager->setLoutres(loutre);
	collisionManager->setLoutreNumber(difficulty+10);
	
	// on replace le joueur en face de l'immeuble
	RULES_Vector pos;
	pos.setCoord(-loutreManager->getImmeuble()->getModel()->getRadius(),joueur->getPos().getY(),0);
	joueur->setPos(pos);
	joueur->setAngleY(-(3.14/2));
	paused = false;
	// et on stop le motion blur
	this->motionBlur->stop();
	
	//init pour le score
	startTimeForScore = glutGet(GLUT_ELAPSED_TIME);
	nbKilledLoutreForScore=0;
	nbShoot=0;
	nbGoodShoot=0;
	hasJustFired = true;
	
	//radar
	hud_balayeur_alpha=0;
	hud_balayeur_deltaAlpha=0.1;

	//on release toutes les touches potentiellement appuy par le joueur a la fin
	// de la partie precedente
	joueur->keyReleased('z');
	joueur->keyReleased('s');
	joueur->keyReleased('q');
	joueur->keyReleased('d');
}

Game::~Game()
{
	// suppression de toutes les variables alloue dynamiquement.
	// on verifie a chaque fois a cause d'un probleme de release probablement
	// due a l'utilisation du pattern singleton
	if (glow != NULL)
	{
		delete glow;
		glow = NULL;
	}
	if (hud_rbalayeur != NULL)
	{
		delete hud_rbalayeur;
		hud_rbalayeur = NULL;
	}
	if (hud_rpoint != NULL)
	{
		delete hud_rpoint;
		hud_rpoint = NULL;
	}
	if (hud_radar != NULL)
	{
		delete hud_radar;
		hud_radar = NULL;
	}
	if (joueur != NULL)
	{
		delete joueur;
		joueur = NULL;
	}
	if (drawer != NULL)
	{
		delete drawer;
		drawer = NULL;
	}
	if (lightMap != NULL)
	{
		delete lightMap;
		lightMap = NULL;
	}
	if (toon != NULL)
	{
		delete toon;
		toon = NULL;
	}
	if (sol != NULL)
	{
		delete sol;
		sol = NULL;
	}
	if( lightPos != NULL)
	{
		delete lightPos;
		lightPos = NULL;
	}
	if (loutreModel != NULL)
	{
		delete loutreModel;
		loutreModel = NULL;
	}
	if (loutreManager != NULL)
	{
		delete loutreManager;
		loutreManager = NULL;
	}
	if (immeubleModel != NULL)
	{
		delete immeubleModel;
		immeubleModel = NULL;
	}
	if (skyBox != NULL)
	{
		delete skyBox;
		skyBox = NULL;
	}
	gluDeleteQuadric(limite);
	if (texFont != NULL)
	{
		delete texFont;
		texFont = NULL;
	}
	if (motionBlur != NULL)
	{
		delete motionBlur;
		motionBlur = NULL;
	}
	if (collisionManager != NULL)
	{
		delete collisionManager;
		collisionManager = NULL;
	}
	if (frustum != NULL)
	{
		delete frustum;
		frustum = NULL;
	}
	if (font != NULL)
	{
		delete font;
		font = NULL;
	}
	if (toon_glow != NULL)
	{
		delete toon_glow;
		toon_glow = NULL;
	}
	
}

void Game::draw(void)
{
	// on nettoie le buffer de profondeur (pas besoin de supprimer celui de couleur, la skybox va tout
	// recouvrir...
	glClearColor (0.0f,0.0f,0.0f,0);
	glClear(GL_DEPTH_BUFFER_BIT);
	float lightRadius2;
	
	// on place la camera du joueur
	glLoadIdentity();
	joueur->look();
	
	///////////////////**************on calcul le blur**************//////////////////////
	if( glowLevel>=1)
	{
		// on initialise le glow (mise en place du viewport...)
		glow->initGlowDrawing();
		// on nettoie le buffer de couleur (c'est plus rapide de la nettoyer maintnant que precedamant car la buffer
		// est plus petit
		glClear(GL_COLOR_BUFFER_BIT);
		
		//////////on rend ici les objets du glow
	
		//on met en place la texture toon spcial pour le glow
		((AbstractDrawer*)(this->drawer))->setTexToonDiffuseID(toon_glow->getId());
		
		/////les loutres (celle de l'image prcdente
		if(glowLevel==2)
		{
			// si le niveau de detail du glow est a 2, on dessine aussi les loutres dans le glow
			Loutre * loutres=loutreManager->getLoutre(0);
			for(int i=0;i<difficulty+10;i++)
			{	
				if(loutres[i].getHasJustBeenDrawn())
				{
					// on ne veut dessiner les loutres qu'une fois par image
					// mise en place d'une couleur particuliere pour les loutres.
					glColor3f(0.729/1.3,0.56/1.3,0.667/1.3);
					loutres[i].draw(true);
				}
			}
		}
		
		/////l'arme est dessine
		glColor3f(0.5,0.5,0.5);
		joueur->draw(true);
		
		/////l'immeuble(en mode normal)
		glColor3f(1,1,1);
		this->drawer->draw(loutreManager->getImmeuble()->getModel(),*lightPos);
		
		/////le sol
		glColor3f(.35,.35,.35);
		lightRadius2 = lightRadius / 2;
		glBindTexture(GL_TEXTURE_2D,lightMap->getId());
		glTexEnvf(GL_TEXTURE_ENV,GL_TEXTURE_ENV_MODE,GL_MODULATE);
		glEnable(GL_BLEND);
		glDepthFunc(GL_LEQUAL);
		glBlendFunc(GL_ONE,GL_ONE);
		// le sol n'est qu'un grand quad textur.
		glBegin(GL_QUADS);
		glTexCoord2f( (-10000-lightPos->getX()+lightRadius2)/lightRadius ,(-10000-lightPos->getZ()+lightRadius2)/lightRadius );
		glVertex3f(-10000,0,-10000);
		glTexCoord2f( (10000-lightPos->getX()+lightRadius2)/lightRadius ,(-10000-lightPos->getZ()+lightRadius2)/lightRadius );
		glVertex3f(10000,0,-10000);
		glTexCoord2f( (10000-lightPos->getX()+lightRadius2)/lightRadius ,(10000-lightPos->getZ()+lightRadius2)/lightRadius );
		glVertex3f(10000,0,10000);
		glTexCoord2f( (-10000-lightPos->getX()+lightRadius2)/lightRadius ,(10000-lightPos->getZ()+lightRadius2)/lightRadius );
		glVertex3f(-10000,0,10000);
		if (joueur->isShooting())
		{
			// si le joueur tir, on affiche aussi un petit cercle de lumiere dessous lui...
			glColor3f(0.95,0.2,0.2);
			glTexCoord2f( (-10000-joueur->getPos().getX()+90)/180 ,(-10000-joueur->getPos().getZ()+90)/180 );
			glVertex3f(-10000,0,-10000);
			glTexCoord2f( (10000-joueur->getPos().getX()+90)/180 ,(-10000-joueur->getPos().getZ()+90)/180 );
			glVertex3f(10000,0,-10000);
			glTexCoord2f( (10000-joueur->getPos().getX()+90)/180 ,(10000-joueur->getPos().getZ()+90)/180 );
			glVertex3f(10000,0,10000);
			glTexCoord2f( (-10000-joueur->getPos().getX()+90)/180 ,(10000-joueur->getPos().getZ()+90)/180 );
			glVertex3f(-10000,0,10000);
		}
		glEnd();
		
		// on finit de calculer le glow (phase de blur)
		glow->computeGlow();
		glClear(GL_DEPTH_BUFFER_BIT);
		// et on termine la gestion du glow
		glow->finishGlowDrawing();
		
		//on remet en place la texture toon par dfaut
		((AbstractDrawer*)(this->drawer))->setTexToonDiffuseID(toon->getId());
	}
	//////////////////////////////////////////////////////////////////////////////////////
	
	
	
	glEnable(GL_TEXTURE_2D);
	glColor3f(0.5,0.5,0.5);
	// on dessine la skybox
	skyBox->drawSkyBox(joueur->getPos().getX(),joueur->getPos().getY(),joueur->getPos().getZ());
	
	///////////////////dessine le sol//////////////////////
	
	glDisable(GL_TEXTURE_2D);
	// le sol est au moins un peu eclair partout, donc on affiche un premier polygon 
	// legerement color...
	glColor3f(0.05,0.05,0.05);
	glBegin(GL_QUADS);
	glVertex3f(-10000,0,-10000);
	glVertex3f(10000,0,-10000);
	glVertex3f(10000,0,10000);
	glVertex3f(-10000,0,10000);
	glEnd();
	glEnable(GL_TEXTURE_2D);
	
	
	glColor3f(.95,.8,.8);
	glBindTexture(GL_TEXTURE_2D,lightMap->getId());
	glTexEnvf(GL_TEXTURE_ENV,GL_TEXTURE_ENV_MODE,GL_MODULATE);
	glEnable(GL_BLEND);
	glDepthFunc(GL_LEQUAL);
	glBlendFunc(GL_ONE,GL_ONE);
	// on affiche la lumiere du sol
	glBegin(GL_QUADS);
	glTexCoord2f( (-10000-lightPos->getX()+lightRadius2)/lightRadius ,(-10000-lightPos->getZ()+lightRadius2)/lightRadius );
	glVertex3f(-10000,0,-10000);
	glTexCoord2f( (10000-lightPos->getX()+lightRadius2)/lightRadius ,(-10000-lightPos->getZ()+lightRadius2)/lightRadius );
	glVertex3f(10000,0,-10000);
	glTexCoord2f( (10000-lightPos->getX()+lightRadius2)/lightRadius ,(10000-lightPos->getZ()+lightRadius2)/lightRadius );
	glVertex3f(10000,0,10000);
	glTexCoord2f( (-10000-lightPos->getX()+lightRadius2)/lightRadius ,(10000-lightPos->getZ()+lightRadius2)/lightRadius );
	glVertex3f(-10000,0,10000);
	if (joueur->isShooting())
	{
		// on affiche la lumiere dessous le joueur quand il tir
		glColor3f(0.95,0.2,0.2);
		glTexCoord2f( (-10000-joueur->getPos().getX()+90)/180 ,(-10000-joueur->getPos().getZ()+90)/180 );
		glVertex3f(-10000,0,-10000);
		glTexCoord2f( (10000-joueur->getPos().getX()+90)/180 ,(-10000-joueur->getPos().getZ()+90)/180 );
		glVertex3f(10000,0,-10000);
		glTexCoord2f( (10000-joueur->getPos().getX()+90)/180 ,(10000-joueur->getPos().getZ()+90)/180 );
		glVertex3f(10000,0,10000);
		glTexCoord2f( (-10000-joueur->getPos().getX()+90)/180 ,(10000-joueur->getPos().getZ()+90)/180 );
		glVertex3f(-10000,0,10000);
	}
	glEnd();
	
	glBlendFunc(GL_DST_COLOR,GL_SRC_COLOR);
	glBindTexture(GL_TEXTURE_2D,sol->getId());
	glTexEnvf(GL_TEXTURE_ENV,GL_TEXTURE_ENV_MODE,GL_REPLACE);
	// on affiche le sol final par lightmapping
	glBegin(GL_QUADS);
	glTexCoord2f(0,0);
	glVertex3f(-10000,0,-10000);
	glTexCoord2f(200,0);
	glVertex3f(10000,0,-10000);
	glTexCoord2f(200,200);
	glVertex3f(10000,0,10000);
	glTexCoord2f(0,200);
	glVertex3f(-10000,0,10000);
	glEnd();
	
	glDisable(GL_BLEND);
	glDepthFunc(GL_LESS);
	/////////////////////////////////////////////////////////
	
	glColor3f(1,1,1);
	// essai de polygon affset pour reduire les probleme de clignotement au loin (en fait ca ne resoud rien...)
	glEnable(GL_POLYGON_OFFSET_LINE);
	glPolygonOffset(1,1);
	// on gere les collision pour placer les loutres au bon endroit
	collisionManager->manageCollision();
	// on gere les loutres.
	loutreManager->manageLoutres(this->difficulty);
	glDisable(GL_POLYGON_OFFSET_LINE);
	// on affiche l'arme du joueur
	joueur->draw(false);
	
	// on affiche la limite du terrain
	glDisable(GL_TEXTURE_2D);
	glEnable(GL_BLEND);
	glBlendFunc(GL_ONE_MINUS_SRC_ALPHA, GL_SRC_ALPHA);
	glDepthMask(GL_FALSE);
	glPushMatrix();
	glRotated(-90,1,0,0);
	glColor4f(1,0.8,0.8,0.8);
	gluCylinder(limite, this->limiteRadius+35, this->limiteRadius+35, 100, 20, 2);
	glPopMatrix();
	glDepthMask(GL_TRUE);
	if (loutreManager->collisionWithImmeuble())
	{
		// il y a eut collision d'une loutre avec l'immeuble
		// on demarre le motion blur
		motionBlur->start();
	}
	
	
	glEnable(GL_TEXTURE_2D);
	// on dessine le motion blur (sans effet si il est arret)
	motionBlur->draw();
	glEnable(GL_BLEND);
	glBlendFunc(GL_SRC_ALPHA,GL_SRC_ALPHA);
	// on affiche le text d'info pour le joueur
	glColor3f(1,1,1);
	font->glText2d(0,0, "score : %d",joueur->getScore());
	font->glText2d(0,15,"vie   : %d",loutreManager->getImmeuble()->getVie());
	if (loutreManager->getImmeuble()->getVie() < 10)
	{
		// on afiche le message warning si le niveau de vie est < 10.
		glColor3f(warningValue[0],warningValue[1],warningValue[2]);
		font->glText2d(365,263,"WARNING");
	}
	glColor3f(1,1,1);
	glDisable(GL_BLEND);
	
	///////////////////**************on rend le blur**************//////////////////////
	if(glowLevel>=1)
	{
		// on affiche le glow en addition a la scene
		glow->renderGlow();
	}
	////////////////////////////////////////////////////////////////////////////////////
	
	// on affiche le radar
	renderRadar();
	
	glFlush();
	glutSwapBuffers();
}


void Game::idle(void)
{
	// on calcule le temps ecoul
	this->calcElapsedTime();
	// on deplace le loueur en fonction des touches qu'il presse
	joueur->processMove(this->elapsedTime);
	// si le joueur sort du terrain
	if (joueur->getPos().length() > this->limiteRadius)
	{
		// le joueur veut sortire de la limite du terrain, on le replace.
		RULES_Vector newPos = joueur->getPos();
		newPos -= joueur->getDirection();
		newPos.normalisation();
		newPos = newPos * this->limiteRadius;
		newPos.setOneCoord(1,joueur->getPos().getY());
		joueur->setPos(newPos);
	}
	// on fait flasher le warning
	warningValue[0] = warningValue[0] + 0.25*(this->elapsedTime*0.001) * warningFactor[0];
	warningValue[1] = warningValue[1] + 0.33*(this->elapsedTime*0.001) * warningFactor[1];
	warningValue[2] = warningValue[2] + 0.40*(this->elapsedTime*0.001) * warningFactor[2];
	for (int i = 0; i < 3; i++)
	{
		if (warningValue[i] < 0)
		{
			warningFactor[i] = 1;
		}
		if (warningValue[i] > 1)
		{
			warningFactor[i] = -1;
		}
	}
	// on update le motion blur
	motionBlur->calcFactor(this->elapsedTime*0.001);
	// on fait tourner la position de la lumiere
	this->lightPos->rotate(0.001*(this->elapsedTime),0,1,0);
	// on force la souris au milieu de l'ecran
	glutWarpPointer(mouseCenterX, mouseCenterY);
	// on verifie la musique
	SoundManager::getInstance()->checkSound();
	
	//on affect au joueur sont score
	joueur->setScore( (glutGet(GLUT_ELAPSED_TIME)-startTimeForScore)/1000 + nbKilledLoutreForScore*(15+this->difficulty) + nbGoodShoot*( (float)this->difficulty/5) );
	
	// update du radar
	hud_balayeur_alpha -= hud_balayeur_deltaAlpha * (this->elapsedTime);

	if (this->loutreManager->getImmeuble()->getVie() <=0)
	{
		// on est morts
		// passage a l'ecran de fin
		EndGLUTMenu * finMenu = EndGLUTMenu::getInstance();
		*(this->currentGlut) = finMenu; 
		finMenu->setCurrentGlut(this->currentGlut);
		// on donne les variables utilise pour le calcule du score final
		finMenu->setScore(this->joueur->getScore());
		finMenu->setAccuracy(((float)this->nbGoodShoot / (float)this->nbShoot)*100);
		finMenu->setKilledLoutre(nbKilledLoutreForScore);
		this->paused=false;
	}
	
	glutPostRedisplay();
	
}


void Game::mouseClicked(int boutton,int etat,int x,int y)
{
	if( (boutton==GLUT_LEFT) && (etat==GLUT_DOWN) )
	{
		// on fait tirer le joueur
		joueur->shoot();
		this->shooting();
		hasJustFired = true;
	}
}

void Game::mouseMoved(int x,int y)
{
	// la souris a bouge, on calcule le nouvel angle de camera
	double deltaX=-(mouseCenterY-y);
	double deltaY=(mouseCenterX-x);
	joueur->addAngleY(deltaY*joueur->getMouseSensibilityY()*(joueur->getZoomLevel()/70));
	joueur->addAngleX(mouseFactor*deltaX*joueur->getMouseSensibilityX()*(joueur->getZoomLevel()/70));
	if (joueur->getAngleX()>=((8*3.14)/18))
	{
		joueur->setAngleX((8*3.14)/18);
	}
	if (joueur->getAngleX()<=(-(8*3.14)/18))
	{
		joueur->setAngleX(-(8*3.14)/18);
	}
	mouseX = x;
	mouseY = y;
}

void Game::mouseMovedClicked(int x,int y)
{
	// la souris a bouge, on calcule le nouvel angle de camera
	double deltaX=-(mouseCenterY-y);
	double deltaY=(mouseCenterX-x);
	joueur->addAngleY(deltaY*joueur->getMouseSensibilityY()*(joueur->getZoomLevel()/70));
	joueur->addAngleX(mouseFactor*deltaX*joueur->getMouseSensibilityX()*(joueur->getZoomLevel()/70));
	if (joueur->getAngleX()>=((8*3.14)/18))
	{
		joueur->setAngleX((8*3.14)/18);
	}
	if (joueur->getAngleX()<=(-(8*3.14)/18))
	{
		joueur->setAngleX(-(8*3.14)/18);
	}
}



void Game::keyPressed(unsigned char key,int x,int y)
{
	// on envoi la touche au joueur qui s'en occupe
	joueur->keyPressed(key);
	switch (key)
	{
	case 27:
		// on veut retourner au menu principal
		this->paused = true;
		glViewport(0,0,1024,768);
		glMatrixMode(GL_PROJECTION);
		glLoadIdentity();
		gluPerspective(90,1024/768,1,10000);
		glMatrixMode(GL_MODELVIEW);
		*(this->currentGlut) = MainMenu::getInstance();
		break;
		
	}
}

void Game::keyReleased(unsigned char key,int x,int y)
{
	// on vient de relacher une touche, on le signal au joueur
	joueur->keyReleased(key);
}

void Game::specialKeyFunc(int key,int x,int y)
{
	// gestion des fleches pour le deplacement
	switch(key)
	{
	case GLUT_KEY_UP:
		joueur->keyPressed(joueur->avance);
		break;
	case GLUT_KEY_DOWN:
		joueur->keyPressed(joueur->recule);
		break;
	case GLUT_KEY_LEFT:
		joueur->keyPressed(joueur->strafeGauche);
		break;
	case GLUT_KEY_RIGHT:
		joueur->keyPressed(joueur->strafeDroite);
		break;

	}
}

void Game::specialKeyUpFunc(int key,int x,int y)
{
	// gestion des fleches pour le deplacement
	switch(key)
	{
	case GLUT_KEY_UP:
		joueur->keyReleased(joueur->avance);
		break;
	case GLUT_KEY_DOWN:
		joueur->keyReleased(joueur->recule);
		break;
	case GLUT_KEY_LEFT:
		joueur->keyReleased(joueur->strafeGauche);
		break;
	case GLUT_KEY_RIGHT:
		joueur->keyReleased(joueur->strafeDroite);
		break;

	}
}

void Game::reshape(int largeur,int hauteur)
{
	// redimentionnement de la fenetre
	glViewport(0,0,(GLsizei) largeur,(GLsizei) hauteur);
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	gluPerspective(90,largeur/hauteur,1,10000);
	glMatrixMode(GL_MODELVIEW);
	
}


Game * Game::getGame()
{
	// pattern singleton
	static Game partie;
	return &partie;
}

void Game::setDifficulty(int diff)
{
	this->difficulty = diff;
}

int Game::getDifficulty()
{
	return this->difficulty;
}


Model * Game::getLoutreModel()
{
	return this->loutreModel;
}

ModelDrawer * Game::getDrawer()
{
	return this->drawer;
}

bool Game::isPaused()
{
	return this->paused;
}

void Game::resetTime()
{
	this->calcElapsedTime();
	this->loutreManager->resetTime();
}

void Game::shooting()
{
	if(hasJustFired)	//on ne va test les tires que si le joueur vient juste de tir!!! et oui
	{
		// incrementation pour le score final
		nbShoot++;
		/*** mise en place de la projection pour le test frustum ***/
		glMatrixMode(GL_PROJECTION);
		glPushMatrix();
		glLoadIdentity();
		// l'angle de vision est tout petit pour simuler un tir precis de pres mais moins de loin
		gluPerspective(0.001,1,1,10000);	//1 degr
		glMatrixMode(GL_MODELVIEW);
		glPushMatrix();
		glLoadIdentity();
		// on place la camera
		joueur->look();
		
		Loutre *loutre = loutreManager->getLoutre(0);
		int KilledLoutre = -1;
		unsigned int distanceIm = (joueur->getPosition() - loutreManager->getImmeuble()->getPosition()).length();
		unsigned int distanceMin = 100000;
		
		//test de l'immeuble avec picking(pas terrible mais obligatoire)
		unsigned int buffer[8];
		glSelectBuffer(8,&buffer[0]);
		int viewport[4];
		glGetIntegerv(GL_VIEWPORT, viewport);
		glMatrixMode(GL_PROJECTION);
		glPushMatrix();
			// on passe en mode selection
			glRenderMode(GL_SELECT);
			glLoadIdentity();
			// initialisation de la matrice de picking
			gluPickMatrix( (GLdouble)512, (GLdouble)viewport[3]-384, 1, 1,viewport);
			gluPerspective(0.001,1,1,10000);
			// on extrait le frustum pour la suite de test (avec la matrice de projection tres serre)
			frustum->Extrait(true);
			glMatrixMode(GL_MODELVIEW);
				glInitNames();
				// on initialise le picking pour l'immeuble
				glPushName(0);
				glDepthFunc(GL_LEQUAL);
				// on dessine l'immeuble pour savoir si on vient de tirer dessus
				this->drawer->drawSimple(loutreManager->getImmeuble()->getModel());
				glDepthFunc(GL_LESS);
				int hits = glRenderMode(GL_RENDER);
			glMatrixMode(GL_PROJECTION);
		glPopMatrix();
		glMatrixMode(GL_MODELVIEW);
		glFlush();
		
		//teste des loutres avec fustrum culling
		for(int i=0;i<difficulty+10 ;i++)
		{
			if( loutre[i].getHasJustBeenDrawn() && loutre[i].isAlive() && frustum->ObjetInFrustum(&loutre[i],loutre[i].getSize()) )
			{
				// pour chaque loutre vivante, dans le frustum et ayant t dessine precedament (comme
				// ca on elimine une partie des loutres (celle qui ne sont mme pas visibles  l'ecran)
				// on calcule sa distance par raport au joueur
				int distance = (joueur->getPosition() - loutre[i].getPosition()).length();
				
				if(distance<distanceMin)
				{
					// elle est la plus proche teste jusqu'a present, c'est donc
					// potentiellement elle qui va etre touche
					distanceMin = distance;
					KilledLoutre = i;
				}
			}
		}
		
		//on modifie la loutre qui a t touch si il y en a une
		if(KilledLoutre != -1)
		{
			if(hits == 0 || distanceIm > distanceMin)
			{
				// le tire a touch une loutre
				nbGoodShoot++;
				// on recalcule la distance (pas top optimise tout ca...)
				float distanceL = (joueur->getPosition() - loutre[KilledLoutre].getPosition()).length();
				// on calcule le facteur de projection de la loutre en fonction de la distance par rapport au joueur
				float coefFactor = (DISTANCE_MAX_SHOOT-distanceL)/DISTANCE_MAX_SHOOT;
				// la loutre perd de la vie
				loutre[KilledLoutre].setVie(loutre[KilledLoutre].getVie() - 1);
				// on lui applique la force de projection.
				loutre[KilledLoutre].setCollision_FactorPushing((3+(2-(this->difficulty)/5)*0.5)*coefFactor);
				// on calcule la direction dans laquelle elle sera projete...
				RULES_Vector tmp;
				// ... en fonction de sa position par rapport au joueur
				tmp = loutre[KilledLoutre].getPosition() - joueur->getPosition();
				// cette force est toujour parallele au sol
				tmp.setOneCoord(1, 0);
				// on renomalise le tout
				tmp.normalisation();
				// et on l'applique a la loutre
				loutre[KilledLoutre].setCollision_ForcePushing(&tmp);
				
				//ajout d'une loutre morte si il y a pour le score
				if(!loutre[KilledLoutre].isAlive())
				{
					nbKilledLoutreForScore++;
				}
			}
		}
		
		glMatrixMode(GL_PROJECTION);
		glPopMatrix();
		glMatrixMode(GL_MODELVIEW);
		glPopMatrix();
	}
	// le tir est effectu, on ne doit plus l'effectuer (sinon on risque de tuer une loutre en 1 tir a chaque fois...)
	hasJustFired = false;
}


void Game::renderRadar()
{
	int i;
	float dx,dz,r;
	glEnable(GL_TEXTURE_2D);
	//mise en place d'une projection orthogonal
	glMatrixMode(GL_PROJECTION);
	glPushMatrix();
	glLoadIdentity();
	gluOrtho2D(0,400,0,300);
	//matrice de transformation d'identit
	glMatrixMode(GL_MODELVIEW);
	glPushMatrix();
	glLoadIdentity();
	
	//on dsactive le depth test
	glDisable(GL_DEPTH_TEST);
	
	//activation du blending en ajout
	glEnable(GL_BLEND);
	glBlendFunc(GL_ONE,GL_ONE );
	
	///////dessin du radar///////
	glBindTexture(GL_TEXTURE_2D,hud_radar->getId());
	glTexEnvf(GL_TEXTURE_ENV,GL_TEXTURE_ENV_MODE,GL_REPLACE);
	glBegin(GL_QUADS);
	glTexCoord2f(0,0);
	glVertex2f(RADAR_MIDDLE_X-RADAR_SIZE,RADAR_MIDDLE_Z-RADAR_SIZE);
	glTexCoord2f(1,0);
	glVertex2f(RADAR_MIDDLE_X+RADAR_SIZE,RADAR_MIDDLE_Z-RADAR_SIZE);
	glTexCoord2f(1,1);
	glVertex2f(RADAR_MIDDLE_X+RADAR_SIZE,RADAR_MIDDLE_Z+RADAR_SIZE);
	glTexCoord2f(0,1);
	glVertex2f(RADAR_MIDDLE_X-RADAR_SIZE,RADAR_MIDDLE_Z+RADAR_SIZE);
	glEnd();
	
	///////dessin de loutre///////
	Loutre * loutre=loutreManager->getLoutre(0);
	glBindTexture(GL_TEXTURE_2D,hud_rpoint->getId());
	glTexEnvf(GL_TEXTURE_ENV,GL_TEXTURE_ENV_MODE,GL_MODULATE);
	glColor3f(0.729,0.56,0.667);
	glBegin(GL_QUADS);
	for(i=0;i<loutreManager->getLoutreNumber();i++)
	{
		//calcul des coordonnes relatives
		RULES_Vector v = loutre[i].getPosition() - joueur->getPosition();
		dx = v.getCoord(0);
		dz = v.getCoord(2);
		
		if( dx>RADAR_REAL_LIMIT || dx<-RADAR_REAL_LIMIT || dz>RADAR_REAL_LIMIT || dz<-RADAR_REAL_LIMIT )
		{
			continue;
		}
		else
		{
			float size = loutre[i].getSize();
			v.rotate(-joueur->getAngleY(),false,true,false);
			dx = -v.getCoord(0);
			dz = v.getCoord(2);
			
			glTexCoord2f(0,0);
			glVertex2f(RADAR_MIDDLE_X+(dx/RADAR_ECHELLE)-RADAR_POINTSIZE*size,RADAR_MIDDLE_Z+(dz/RADAR_ECHELLE)-RADAR_POINTSIZE*size);
			glTexCoord2f(1,0);
			glVertex2f(RADAR_MIDDLE_X+(dx/RADAR_ECHELLE)+RADAR_POINTSIZE*size,RADAR_MIDDLE_Z+(dz/RADAR_ECHELLE)-RADAR_POINTSIZE*size);
			glTexCoord2f(1,1);
			glVertex2f(RADAR_MIDDLE_X+(dx/RADAR_ECHELLE)+RADAR_POINTSIZE*size,RADAR_MIDDLE_Z+(dz/RADAR_ECHELLE)+RADAR_POINTSIZE*size);
			glTexCoord2f(0,1);
			glVertex2f(RADAR_MIDDLE_X+(dx/RADAR_ECHELLE)-RADAR_POINTSIZE*size,RADAR_MIDDLE_Z+(dz/RADAR_ECHELLE)+RADAR_POINTSIZE*size);
		}
	}
	glEnd();
	
	///////dessin de l'immeuble///////
	glTexEnvf(GL_TEXTURE_ENV,GL_TEXTURE_ENV_MODE,GL_MODULATE);
	glBegin(GL_QUADS);
	//calcul des coordonnes relatives
	RULES_Vector v = joueur->getPosition();
	v*=-1;
	dx = v.getCoord(0);
	dz = v.getCoord(2);
	r = loutreManager->getImmeuble()->getModel()->getRadius()/RADAR_ECHELLE;
	
	if( !(dx>RADAR_REAL_LIMIT || dx<-RADAR_REAL_LIMIT || dz>RADAR_REAL_LIMIT || dz<-RADAR_REAL_LIMIT) )
	{
		v.rotate(-joueur->getAngleY(),false,true,false);
		dx = -v.getCoord(0);
		dz = v.getCoord(2);
		
		glColor3f(1,0,0);
		glTexCoord2f(0,0);
		glVertex2f(RADAR_MIDDLE_X+(dx/RADAR_ECHELLE)-r,RADAR_MIDDLE_Z+(dz/RADAR_ECHELLE)-r);
		glTexCoord2f(1,0);
		glVertex2f(RADAR_MIDDLE_X+(dx/RADAR_ECHELLE)+r,RADAR_MIDDLE_Z+(dz/RADAR_ECHELLE)-r);
		glTexCoord2f(1,1);
		glVertex2f(RADAR_MIDDLE_X+(dx/RADAR_ECHELLE)+r,RADAR_MIDDLE_Z+(dz/RADAR_ECHELLE)+r);
		glTexCoord2f(0,1);
		glVertex2f(RADAR_MIDDLE_X+(dx/RADAR_ECHELLE)-r,RADAR_MIDDLE_Z+(dz/RADAR_ECHELLE)+r);
	}
	glEnd();
	
	///////dessin du balayeur///////
	glBlendFunc(GL_ONE,GL_ONE);
	glBindTexture(GL_TEXTURE_2D,hud_rbalayeur->getId());
	glTexEnvf(GL_TEXTURE_ENV,GL_TEXTURE_ENV_MODE,GL_REPLACE);
	glMatrixMode(GL_TEXTURE);
	glPushMatrix();
	glLoadIdentity();
	glTranslatef(0.5,0.5,0);
	glRotatef(hud_balayeur_alpha,0,0,1);
	glMatrixMode(GL_MODELVIEW);
	glBegin(GL_QUADS);
	glTexCoord2f(-0.5,-0.5);
	glVertex2f(RADAR_MIDDLE_X-RADAR_SIZE,RADAR_MIDDLE_Z-RADAR_SIZE);
	glTexCoord2f(0.5,-0.5);
	glVertex2f(RADAR_MIDDLE_X+RADAR_SIZE,RADAR_MIDDLE_Z-RADAR_SIZE);
	glTexCoord2f(0.5,0.5);
	glVertex2f(RADAR_MIDDLE_X+RADAR_SIZE,RADAR_MIDDLE_Z+RADAR_SIZE);
	glTexCoord2f(-0.5,0.5);
	glVertex2f(RADAR_MIDDLE_X-RADAR_SIZE,RADAR_MIDDLE_Z+RADAR_SIZE);
	glEnd();
	glMatrixMode(GL_TEXTURE);
	glPopMatrix();
	glMatrixMode(GL_MODELVIEW);
	
	//dsactivation du blending
	glDisable(GL_BLEND);
	
	//on ractive le depth test
	glEnable(GL_DEPTH_TEST);
	
	//on restore la projection original
	glMatrixMode(GL_PROJECTION);
	glPopMatrix();
	//on restore la modelview original
	glMatrixMode(GL_MODELVIEW);
	glPopMatrix();
	
}

void Game::setPaused(bool pause)
{
	this->paused = pause;
}

Joueur * Game::getPlayer()
{
	return this->joueur;
}

void Game::loadRessources(int texQuality, int filtrageQuality, bool loutreLow)
{
	// on determine le filtrage  utiliser pour le chargement des textures en fonctions du niveau de filtrage pass
	// en parametre...
	int magFilter = GL_NEAREST;
	int minFilter = GL_NEAREST;
	switch (filtrageQuality)
	{
	case 1:
		magFilter = GL_LINEAR;
		minFilter = GL_LINEAR;
		break;
	case 2:
		magFilter = GL_LINEAR;
		minFilter = GL_LINEAR_MIPMAP_NEAREST;
		break;
	case 3:
		magFilter = GL_LINEAR;
		minFilter = GL_LINEAR_MIPMAP_LINEAR;
		break;
	}
	// on verifie que la qualit des textures ne sorte pas des bornes
	if (texQuality < 0)
	{
		texQuality = 0;
	}
	if (texQuality > 3)
	{
		texQuality = 3;
	}
	// on charge le sky box
	skyBox->back->loadBMP("./ressources/textures/skybox/back.bmp",true,magFilter,minFilter,GL_CLAMP,GL_CLAMP,texQuality,false,0,0,255,0,1);
	skyBox->bottom->loadBMP("./ressources/textures/skybox/bottom.bmp",true,magFilter,minFilter,GL_CLAMP,GL_CLAMP,texQuality,false,0,0,255,0,1);
	skyBox->front->loadBMP("./ressources/textures/skybox/front.bmp",true,magFilter,minFilter,GL_CLAMP,GL_CLAMP,texQuality,false,0,0,255,0,1);
	skyBox->left->loadBMP("./ressources/textures/skybox/left.bmp",true,magFilter,minFilter,GL_CLAMP,GL_CLAMP,texQuality,false,0,0,255,0,1);
	skyBox->right->loadBMP("./ressources/textures/skybox/right.bmp",true,magFilter,minFilter,GL_CLAMP,GL_CLAMP,texQuality,false,0,0,255,0,1);
	skyBox->top->loadBMP("./ressources/textures/skybox/top.bmp",true,magFilter,minFilter,GL_CLAMP,GL_CLAMP,texQuality,false,0,0,255,0,1);
	
	// on charge la texture du sol
	sol->loadBMP("./ressources/textures/sol.bmp",true,magFilter,minFilter,GL_REPEAT,GL_REPEAT,texQuality,false,255,0,255,0,1);
	
	// et la lightmap
	lightMap->loadBMP("./ressources/textures/lightmap2.bmp",false,GL_LINEAR,GL_LINEAR,GL_CLAMP,GL_CLAMP,texQuality,false,0,0,255,0,1);
	
	if (!loutreLow)
	{
		// on supprime le model precedent
		delete loutreModel;
		// et on le recharge en haute resolution
		loutreModel = new Model();
		loutreModel->load("./ressources/models/loutre.obj");
		this->useLoutreLow = false;
		// et on lui assigne sa nouvelle texture
		loutreModel->setTexture(new RULES_Texture());
		for (int i = 0; i < loutreManager->getLoutreNumber();i++)
		{
			// pour chaque loutre, on lui applique son nouveau model
			loutreManager->getLoutre(i)->setModel(loutreModel);
		}
	}
	else
	{
		// on supprime le model precedent
		delete loutreModel;
		// et on le recharge en basse resolution
		loutreModel = new Model();
		loutreModel->load("./ressources/models/loutre_low.obj");
		this->useLoutreLow = false;
		loutreModel->setTexture(new RULES_Texture());
		for (int i = 0; i < loutreManager->getLoutreNumber();i++)
		{
			// on assigne le nouveau model a toutes les loutres
			loutreManager->getLoutre(i)->setModel(loutreModel);
		}
	}
	// on recharge la texture de la loutre
	loutreModel->getTexture()->loadBMP("./ressources/textures/loutre.bmp",true,magFilter,minFilter,GL_CLAMP,GL_CLAMP,texQuality,false,0,0,255,0,1);
	// celle de l'immeuble
	immeubleModel->getTexture()->loadBMP("./ressources/textures/immeuble.bmp",true,magFilter,minFilter,GL_CLAMP,GL_CLAMP,texQuality,false,0,0,255,0,1);
	// et celle de l'arme du joueur
	joueur->getModel()->getTexture()->loadBMP("./ressources/textures/arme.bmp",true,magFilter,minFilter,GL_REPEAT,GL_REPEAT,texQuality,false,255,0,255,0,1);
}

void Game::setGlowLevel(unsigned int glowL)
{
	glowLevel = glowL;
}
