// Glow.h: interface for the Glow class.
//
//////////////////////////////////////////////////////////////////////

#ifndef GLOW_H
#define GLOW_H

#ifdef WIN32
#include <windows.h>
#endif
#include <GL/gl.h>
#include <GL/glut.h>

/**
*	Classe permettant d'appliquer un effet de glow sur une scene
*
* algo de rendu :
*
*	_ initGlowDrawing<br>
*			//dessin de la scene  "glower"<br>
*	_ computeGlow<br>
*	_ finishGlowDrawing<br>
*			//dessin de la scene normal<br>
*	_ renderGlow<br>
*/
class Glow  
{

private:
	/**
	* sauvegarde du viewport a restaurer
	*/
	int tmpViewPort[4];

	/**
	* hauteur du buffer de glow en texel
	*/
	unsigned int glow_height;

	/**
	* largeur du buffer de glow en texel
	*/
	unsigned int glow_width;

	/**
	* nb pass pour glowrer
	*/
	unsigned int glow_nbPass;

	/**
	* force du glow
	*/
	float glow_power;

protected:
	/**
	* l'identifiant de la texture de glow
	*/
	unsigned int idTex;

	/**
	* charge le buffer cran arrire dans la texture
	*/
	void loadPixelIntoTexture();

public:

	/**
	* permet d'accd  la force du glow
	*
	* @return la force du glow
	*/
	float getGlowPower();

	/**
	* permet d'accd au nombre de pass pour le glow
	*
	* @return le nombre de pass pour le glow
	*/
	unsigned int getNbPass();

	/**
	* permet d'accd  la hauteur du buffer de glow en texel
	*
	* @return la hauteur du buffer de glow en texel
	*/
	unsigned int getHeight();

	/**
	* permet d'accd  la largeur du buffer de glow en texel
	*
	* @return la largeur du buffer de glow en texel
	*/
	unsigned int getWidth();

	/**
	*	Regle la force du glow
	* Dtermine la prsence du glow dans la scene final(simple modulate sur la texture)
	*
	* @param la force du glow
	*/
	void setGlowPower(float bp);

	/**
	*	Regle le nombre de pass pour le glow
	* Le nombre de passe dtermine l'etalement du glow autour des objets
	*
	* @param le nombre de pass pour le glow
	*/
	void setNbPass(unsigned int nbp);

	/**
	* Regle la hauteur du buffer de glow en texel
	*
	* @param la hauteur du buffer de glow en texel
	*/
	void setHeight(unsigned int height);

	/**
	* Regle la largeur du buffer de glow en texel
	*
	* @param la largeur du buffer de glow en texel
	*/
	void setWidth(unsigned int width);

	/**
	* Rend le glow  l'cran
	*/
	void renderGlow();

	/**
	*	Calcul le glow.
	* Cette mthode crit dans le buffer arrire de l'cran donc bien prvoir son appel
	* (gneralement aprs avoir rendue la scene a glow)
	*/
	void computeGlow();

	/**
	* Termine le rendu du glow
	* (remet l'ancien viewport)
	*/
	void finishGlowDrawing();

	/**
	* Initialise le rendu du glow
	* (initialise le viewport)
	*/
	void initGlowDrawing();

	/**
	* constructeur
	*
	* REM : plus le buffer est grand, plus il faudra des pass pour taler le glow
	*mais plus la qualit sera meilleur...  vous de rgler
	*
	* @param width la largeur du buffer de glow
	* @param height la hauteur du buffer de glow
	*/
	Glow(unsigned int width,unsigned int height);

	/**
	* destructeur
	*/
	virtual ~Glow();

};

#endif