// Loutre.h: interface for the Loutre class.
//
//////////////////////////////////////////////////////////////////////

/**
 * documentation ralis par Michel de VERDELHAN : michel.deverdelhan@free.fr
 * le 25/10/2004
 */

#if !defined(LOUTRE_H)
#define LOUTRE_H
#define UBERLOUTRE_SIZE 2.5f
#define LOUTRE_MAX_SIZE 1.5f

#include "Objet.h"
#include "RULES_Vector.h"	// Added by ClassView
#include <GL/glut.h>
#include "SoundManager.h"

/**
 * classe representant une loutre du jeu.
 * </BR>une loutre doit apparaitre de facon alatoire sur le terrain et avancer vers l'immeuble.
 * </BR> les loutres sont anim en les faisant osciller le long de leur axe Z.
 */
class Loutre : public Objet 
{
private:
	/**
	 * le temp de depart de l'animation de la loutre. ce temp est determin alatoirement a chaque initialisation
	 * de la loutre pour cre une desinchronisation entre toutes les loutres.
	 */
	int randTime;

	/**
	 * la vitesse de rotation de la loutre autour de l'axe Z de la loutre pour l'animation
	 */
	float angleSpeed;

	/**
	 * pointeur sur la position de la lumiere pour le toon shading
	 */
	RULES_Vector * light;

	/**
	 * retourne l'angle entre 2 vecteurs.
	 */
	float angle(RULES_Vector * vect1,RULES_Vector * vect2);

	/**
	 * l'angle d'incrementation par seconde pour l'animation.
	 */
	float incrAngle;

	/**
	 * l'angle courant d'animation.
	 */
	float currentAngle;

	/**
	 * l'angle maximum de rotation lors de l'animation. l'angle courant est comprit entre [-maxAngle..+maxAngle]
	 */
	float maxAngle;

	/**
	 * la vie de la loutre, determin par le niveau de difficult.
	 */
	int vie;

	/**
	 * la vitesse de la loutre, fonction du niveau de difficul et du nombre de fois ou la loutre
	 * a t tue.
	 */
	float speed;

	/**
	 * la direction dans laquel se deplace la loutre.
	 */
	RULES_Vector direction;

	/**
	 * la destination de la loutre. a chaque iteration de la boucle de jeu, la direction de la loutre tens vers 
	 * la destination.
	 */
	RULES_Vector destination;

	/**
	 * variable du temp precedent, utiliser pour calculer le facteur qui sert pour le deplacement time based
	 */
	int previousTime;

	/**
	 * le facteur de deplacement a chaque iteration de la boucle principal de jeu. cette variable est fonction du temp
	 * ecoul entre 2 iteration.
	 */
	float factor;

	/**
	 * le "taux" de vecteur de collision a ajouter a chaque frame au vecteur de direction
	 */
	float collision_factor;

	/**
	 * le taux de vecteur de collision (lors d'un tir ou ocollision avec le joueur) a ajouter a chaque frame au vecteur de direction
	 */
	float collision_factorPushing;

	/**
	 * le vecteur de deplacement lors d'une collision
	 */
	RULES_Vector *collision_force;

	/**
	 * le vecteur de deplacement lors d'une collision de tir ou c'une collision avec le joueur.
	 */
	RULES_Vector *collision_forcePushing;

	/**
	 * permet de savoir si la loutre vient d'etre dessin (utilis par le glow ambiant)
	 */
	bool hasJustBeenDrawn;

	/** 
	 * la taille de la loutre
	 */
	float size;

	/**
	 *  permet de savoir si la loutre est une ber loutre
	 */
	bool uberLoutre;
public:

	/**
	 * incremente dans la plage valide la taille de la loutre
	 */
	void incrementeSize()
	{
		size += .1f;
		if (size > LOUTRE_MAX_SIZE) size = LOUTRE_MAX_SIZE;
	};

	/**
	 * definit si la loutre est une uber loutre
	 * @param u: definit la uberloutricit de la loutre.... ;-)
	 */
	void setUberLoutre(bool u){uberLoutre = u;};

	/**
	 * permet de savoir si la loutre est une uber loutre
	 * @return bool true si la loutre est une uber loutre
	 */
	bool isUberLoutre(){return uberLoutre;};

	/**
	 * retourne la taille de la loutre
	 * @return float: la taille de la loutre
	 */
	float getSize()
	{
		if (uberLoutre)
		{
			return UBERLOUTRE_SIZE;
		}
		else
		{
			return size;
		}
	}

	/**
	 * permet de savoir si une loutre vient d'etre dessine
	 * @return true: la loutre a deja t dessine
	 */
	bool getHasJustBeenDrawn();

	/**
	 * definit si la loutre a deja t dessine
	 * @param d: true => la loutre a t dessine
	 */
	void setHasJustBeenDrawn(bool d);

	/**
	 * methode permetant de deplacer la loutre en fonction de son vecteur de deplacement et des collisions
	 * qu'elle subit.
	 */
	void moveLoutre();

	/**
	 * definit le vecteur de collision avec le joueur
	 * @param cfp: le vecteur de direction lors d'une collision
	 */
	void setCollision_ForcePushing(RULES_Vector *cfp);

	/**
	 * retourne le vecteur de collision avec le joueur
	 * @return RULES_Vector: le vecteur de collision avec le joueur
	 */
	RULES_Vector* getCollision_ForcePushing();

	/**
	 * definit le vecteur de collision lors d'une collision loutre/loutre
	 * @param cf: le vecteur de collision
	 */
	void setCollision_Force(RULES_Vector *cf);

	/**
	 * retourne de vecteur de collision loutre/loutre
	 * @return RULES_Vector: le vecteur de collision loutre/loutre
	 */
	RULES_Vector* getCollision_Force();

	/**
	 * definit le facteur de collision loutre/loutre au moment de la collision. ce vecteur va decroitre 
	 * au fur et a mesure du temps.
	 * @param cf: le facteur.
	 */
	void setCollision_Factor(float cf);

	/**
	 * retourne de facteur de collision loutre/loutre
	 * @return float: le facteur.
	 */
	float getCollision_Factor();

	/**
	 * definit le facteur de collision loutre/joueur.
	 * @param cfp: le facteur
	 */
	void setCollision_FactorPushing(float cfp);

	/**
	 * retourne de facteur de collision loutre/joueur
	 * @return float: le facteur
	 */
	float getCollision_FactorPushing();

	/**
	 * calcule de facteur qui sera utilis pour effectuer les mouvement en fonction du temps.
	 * cette fonction doit etre appele obligatoirement une et une seule fois par frame. Sinon, le deplacement
	 * des loutre ne s'effectue plus en fonction du temp rel.
	 */
	void calcFactor();

	/**
	 * fonction permettant de savoir si la loutre depasse du sol. une loutre qui deppase du sol est
	 * soit une loutre vivante, soit une loutre morte mais dont l'animation de mort n'est pas encore termine
	 */
	bool isUpToFloor();

	/**
	 * definit la position de la lumiere qui eclairera la loutre avec le toon shading
	 * @param lightPos: la position de la lumiere en position absolue dans l'espace.
	 */
	void setLight(RULES_Vector * lightPos);

	/**
	 * retourne la destination de la loutre
	 * @return RULES_Vector: la destination de la loutre
	 */ 
	RULES_Vector getDestination();

	/**
	 * retourne la vitesse de la loutre.
	 * @return float: la vitesse de la loutre.
	 */
	float getSpeed();

	/**
	 * retourne la vie de la loutre
	 * @return int: la vie de la loutre
	 */
	int getVie();

	/**
	 * retourne l'angle max d'animation de la loutre
	 * @return float: l'angle max d'animation de la loutre.
	 */
	float getMaxAngle();

	/**
	 * retourne l'angle courant d'animation
	 * @return float: l'angle courant.
	 */
	float getCurrentAngle();

	/**
	 * retourne l'angle d'incrementation des animation de la loutre
	 * @return float: l'angle d'incremantation.
	 */
	float getIncrAngle();

	/**
	 * definit la destination de la loutre
	 * @param dest: la destination de la loutre.
	 */
	void setDestination(RULES_Vector dest);

	/**
	 * definit la direction de la loutre. La direction de la loutre est le vecteur
	 * qui est additionn a la position de la loutre a chaque frame.
	 * @param direc: la direction de la loutre.
	 */
	void setDirection(RULES_Vector direc);

	/**
	 * definit la vitesse de la loutre.
	 * @param vitesse: la vitesse de la loutre.
	 */
	void setSpeed(float vitesse);

	/**
	 * definit la vie de la loutre
	 * @param life: la vie de la loutre
	 */
	void setVie(int life);

	/**
	 * definit l'angle max d'animation de la loutre
	 * @param angle: l'angle max d'animation
	 */
	void setMaxAngle(float angle);

	/**
	 * definit l'angle courant d'animation
	 * @param angle; l'angle courant d'animation
	 */
	void setCurrentAngle(float angle);

	/**
	 * definit l'angle d'incrementation a chaque frame d'animation
	 * @param angle: l'angle d'incremantation.
	 */
	void setIncrAngle(float angle);

	/**
	 * calcule la direction de la loutre en fonction de sa destination et des collision qu'elle subit.
	 */
	void calcDirection();

	/**
	 * deplace la loutre en fonction de sa direction et du temp ecoul entre chaque frame.
	 */
	void move();

	/**
	 * permet de savoire si une loutre est vivante ou pas. Une loutre est morte quand sa vie est < 0
	 */
	bool isAlive();

	/**
	 * initialise une loutre. Definit la position de depart de la loutre (aleatoiremant), et les differents
	 * parametres d'animation.
	 * @param center: le centre vers lequel se deplace la loutre.
	 * @param radius: le rayon d'apparition de la loutre. permet de ne pas faire apparaitre de loutre directement
	 * dans la zone de jeu mais a l'exterieur. La distance effective d'apparition de la loutre est choisit aleatoirement
	 * et est > au rayon.
	 */ 
	void initialise(RULES_Vector center,float radius);

	/**
	 * retourne le vecteur de direction de la loutre.
	 * @return RULES_Vector: la direction de la loutre.
	 */
	RULES_Vector getDirectionVector();

	/**
	 * dessine la loutre.
	 * @param onlyToon: permet de savoir si on doit juste dessiner le toon ou la loutre complete (avec la texture)
	 */
	void draw(bool onlyToon);

	/**
	 * constructeur pas defaut
	 */ 
	Loutre();

	/**
	 * destructeur par defaut.
	 */
	virtual ~Loutre();

};

#endif
