// LoutreManager.cpp: implementation of the LoutreManager class.
//
//////////////////////////////////////////////////////////////////////

#include "LoutreManager.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

LoutreManager::LoutreManager()
{
	// on cree un nouvel immeuble
	immeuble = new Immeuble();
	generatedLoutre = 0;
}

LoutreManager::~LoutreManager()
{
	// on detruit les loutre... bien fair attention a ce que les loutres ne soient pas detruites pas un tiers...
	delete [] loutres;
	// on detruit l'immeuble
	delete immeuble;
}

void LoutreManager::manageLoutres(int difficulty)
{
	// il n'y a pas de loutre dans l'immeuble par defaut
	this->loutreOnImmeuble = false;
	// on recupere le frustum de vue
	frustum.Extrait(true);
	// pour chaque loutre
	for (int i = 0; i < this->loutreNumber; i++)
	{
		// on calcule sa direction
		loutres[i].calcDirection();
		// et on effectue son mouvement
		loutres[i].move();
		// on calcule la distance de la loutre avec le centre de l'immeuble
		RULES_Vector tmp = immeuble->getPosition() - loutres[i].getPosition();
		// si la distance loutre/immeuble < rayon de l'immeuble et que la loutre est vivante (on ne test plus les loutre
		// mortes car elle sont consider comme ayant deja touch l'immeuble une fois)
		if ((tmp.length() < ((loutres[i].getModel()->getRadius() * loutres[i].getSize()) + immeuble->getModel()->getRadius())-10)&&loutres[i].isAlive())
		{
			// la loutre touche l'immeuble
			// on retire l'equivalent de la vie de la loutre  l'immeuble
			immeuble->setVie(immeuble->getVie() - loutres[i].getVie());
			// la vie de l'immeuble ne peut etre en dessous de 0
			if (immeuble->getVie() < 0)
			{
				immeuble->setVie(0);
			}
			// et on tue la loutre
			loutres[i].setVie(0);
			// on demarre l'effet de motion blur
			this->loutreOnImmeuble = true;
			// et on joue le son de collision
			SoundManager::getInstance()->playBoom();
		}
		// si la loutres est morte et qu'elle est en dessous du sol (cad qu'elle n'est plus visible)
		if ((!loutres[i].isAlive())&&(!loutres[i].isUpToFloor()))
		{
			// on veut reinitialiser la loutre
			// on effectue une boucle pour verifier que la loutre n'est pas apparue dans une loutre deja existante
			bool test = false;
			do
			{
				test = false;
				// on initialise la loutre
				loutres[i].initialise(loutres[i].getDestination(),this->creationRadius);
				// pour chaque loutre
				for (int l = 0;l < this->loutreNumber;l++)
				{
					// on ne veut pas tester la loutre qu'on vient de faire apparaitre
					// car elle est toujours dedans elle mme ;-)
					if (i != l)
					{
						// on calcule la distance entre les 2 loutres
						float distance = (this->loutres[i].getPosition() - this->loutres[l].getPosition()).length();
						// si la distance entre loutre est inferieur au rayon du model d'une loutre
						if (distance <= loutres[l].getSize()*2)
						{
							// alors il faut reinitialiser la loutre encore une fois car elle n'est pas valide
							test = true;
							break;
						}
					}
				}
			}while (test);
			// on definit la vie de la loutre en fonction du niveau de difficult
			loutres[i].setVie(1+(int)(difficulty/5));
			// on augmente sa vitesse, plus une loutre est morte souvent plus elle va vite
			loutres[i].setSpeed(loutres[i].getSpeed()+0.005);

			if (generatedLoutre >= UBER_LOUTRE_NUMBER)
			{
				// on genere une uber loutre
				loutres[i].setUberLoutre(true);
				loutres[i].setVie(loutres[i].getVie()*((difficulty/3.f) + 1));
				generatedLoutre = 0;
			}
			else
			{
				// on genere une loutre normale
				loutres[i].incrementeSize();
				loutres[i].setUberLoutre(false);
				generatedLoutre++;
			}
			// elle ne subit pas de collision
			loutres[i].setCollision_Factor(1);
			// ni de tir
			loutres[i].setCollision_FactorPushing(0);
		}
		// si la loutre est visible
		if (frustum.ObjetInFrustum(&loutres[i],loutres[i].getSize()))
		{
			// on l'affiche
			glColor3f(1,1,1);
			loutres[i].draw(false);
			//et on signal qu'elle a t affich
			loutres[i].setHasJustBeenDrawn(true);
		}
		else
		{
			// sinon, on signal qu'elle n'a pas t dessin
			loutres[i].setHasJustBeenDrawn(false);
		}
	}
	// si l'immeuble est visible
	if (frustum.ObjetInFrustum(immeuble))
	{
		// on l'affiche
		glColor3f(1,1,1);
		immeuble->draw();
	}

}

Loutre * LoutreManager::getLoutre(int index)
{
	return &loutres[index];
}


Immeuble * LoutreManager::getImmeuble()
{
	return immeuble;
}

void LoutreManager::setCreationRadius(float radius)
{
	this->creationRadius = radius;
}

void LoutreManager::setLoutres(Loutre * loutre,int number)
{
	// on copie le pointeur sur les loutres
	loutres = loutre;
	for (int i = 0; i < number ; i++)
	{
		// et on itinialise chaque loutre
		loutres[i].initialise(loutres[i].getDestination(),this->creationRadius);
		loutres[i].calcFactor();
	}

	this->loutreNumber = number;
}

void LoutreManager::setImmeuble(Immeuble *immeuble)
{
	this->immeuble = immeuble;
}

void LoutreManager::killAll()
{
	for (int i = 0; i < this->loutreNumber ; i++)
	{
		// pour chaque loutre, on la tue
		loutres[i].setVie(0);
	}
}

bool LoutreManager::collisionWithImmeuble()
{
	return this->loutreOnImmeuble;
}

void LoutreManager::resetTime()
{
	for (int i =0; i < this->loutreNumber; i++)
	{
		loutres[i].calcFactor();
	}
}

int LoutreManager::getLoutreNumber()
{
	return this->loutreNumber;
}
