// Model.cpp: implementation of the Model class.
//
//////////////////////////////////////////////////////////////////////

#include "Model.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

Model::Model()
{
	this->vertex = NULL;
	this->triangles = NULL;
	this->vertexNumber = 0;
}

Model::~Model()
{
	delete [] vertex;
	delete [] triangles;
	delete this->texture;
}

void Model::load(char *file)
{
	// on cre le lecteur de .obj
	OBJ_Reader reader(file);

	// on lit le fichier
	reader.readFile();

//reader.print();

	// on recupere le nombre des faces
	this->vertexNumber = reader.getFace().size() * 3 ;

	// on supprime le tableau de triangles si il existe
	if (this->triangles != NULL)
	{
		delete [] this->triangles;
	}

	// on cre les tableaux de triangles
	this->triangles = new int[vertexNumber];

	// on supprime le tableau de vertex si il existe
	if (this->vertex != NULL)
	{
		delete [] this->vertex;
	}

	// on cre le tableau de vertex
	this->vertex = new ModelVertex[vertexNumber];

	// pour chaque face du loader, on lit les indices du vertex, des coordonne de textures et de la
	// normale et on les stockent dans le ModelVertex et on met l'indice du modelVertex dans le
	// tableau de triangle.
	// vertexIndice est l'indice courant dans le tableau de vertex
	int vertexIndice = 0;
	for (int i = 0; i < reader.getFace().size(); i ++)
	{
		// on recupere la face courante
		OBJ_R_Face face = reader.getFace()[i];
		// chaque triangle a 3 vertex
		for (int j = 0; j < 3; j++)
		{
			// je prend le vertex j du la face i et je le met dans le tableau
			OBJ_R_Vertex vert = reader.getVertex()[face.vertex[j]];
			this->vertex[vertexIndice].vertex.setCoord(vert.x,vert.y,vert.z);

			// on prend les coordonne de texture du vertex j de la face i et on les met dans le
			// tableau
			OBJ_R_Texture tex = reader.getTexture()[face.texture[j]];
			this->vertex[vertexIndice].texCoord[0] = tex.u;
			this->vertex[vertexIndice].texCoord[1] = tex.v;

			// on prend la normale du vertex j de la face i et on le met dans le tableau de vertex
			OBJ_R_Vertex norm = reader.getNormal()[face.normal[j]];
			this->vertex[vertexIndice].normal.setCoord(norm.x,norm.y,norm.z);

			// on stock l'indice de ce vertex dans le tableau d'indice des triangles
			this->triangles[vertexIndice] = vertexIndice;
			// on incremente l'indice d'ecriture dans le tableau de vertex
			vertexIndice ++;
		}
	}

	this->transforme();

}

ModelVertex * Model::getVertex()
{
	return vertex;
}

int * Model::getTriangles()
{
	return this->triangles;
}

unsigned int Model::getVertexNumber()
{
	return this->vertexNumber;
}

void Model::transforme()
{
	// il faut mettre le vertex le + bas a la hauteure 0 et 
	// calculer le centre du model sur le plan x,z
	// on recherche le vertex le + bas , le + a gauche, le + a froite,
	// le plus en arriere, le plus en avant
	ModelVertex * lowest = &this->vertex[0];
	ModelVertex * haut = &this->vertex[0];
	ModelVertex * gauche =  &this->vertex[0];
	ModelVertex * droite =  &this->vertex[0];
	ModelVertex * arriere =  &this->vertex[0];
	ModelVertex * avant =  &this->vertex[0];
	int i;
	for (i = 1; i < this->vertexNumber; i++)
	{
		if (this->vertex[i].vertex.getCoord(1) < lowest->vertex.getCoord(1))
		{
			// le vertex i est inferieur en y
			lowest = &vertex[i];
		}
		if (this->vertex[i].vertex.getCoord(1) > haut->vertex.getCoord(1))
		{
			// le vertex i est inferieur en y
			haut = &vertex[i];
		}
		if (this->vertex[i].vertex.getCoord(0) < gauche->vertex.getCoord(0))
		{
			// le vertex i est le plus a gauche
			gauche = &vertex[i];
		}
		if (this->vertex[i].vertex.getCoord(0) > droite->vertex.getCoord(0))
		{
			// le vertex i est le plus a droite
			droite = &vertex[i];
		}
		if (this->vertex[i].vertex.getCoord(2) < arriere->vertex.getCoord(2))
		{
			// le vertex i est le plus vers l'arriere
			arriere = &vertex[i];
		}
		if (this->vertex[i].vertex.getCoord(2) > avant->vertex.getCoord(2))
		{
			// le vertex i est le plus vers l'avant
			avant = &vertex[i];
		}
	}
	// on calcule la difference entre la coordonne y du vertex le plus bas et 0
	float differencey = -lowest->vertex.getCoord(1);
	float differencex = -(droite->vertex.getCoord(0) + gauche->vertex.getCoord(0))/2;
	float differencez = -(avant->vertex.getCoord(2) + arriere->vertex.getCoord(2))/2;
	// et on applique cette difference a tout les vertex
	for (i = 0; i < this->vertexNumber; i++)
	{
		vertex[i].vertex.setCoord(vertex[i].vertex.getCoord(0) + differencex,vertex[i].vertex.getCoord(1) + differencey,vertex[i].vertex.getCoord(2) + differencez);
	}

	// et on calcule le rayon qui est le plus grand de {gauche,droite,avant,arriere}
	radius = fabs(gauche->vertex.getCoord(0));
	if (fabs(droite->vertex.getCoord(0)) > radius)
	{
		radius = fabs(droite->vertex.getCoord(0));
	}
	if (fabs(avant->vertex.getCoord(2)) > radius)
	{
		radius = fabs(avant->vertex.getCoord(2));
	}
	if (fabs(arriere->vertex.getCoord(2)) > radius)
	{
		radius = fabs(arriere->vertex.getCoord(2));
	}

	RULES_Vector centre;
	centre.setCoord(0,haut->vertex.getCoord(1)/2,0);
	this->boundingBox.setAttributs(centre,haut->vertex.getCoord(1)/2,radius,radius);

}

float Model::getRadius()
{
	return this->radius;
}

void Model::setTexture(RULES_Texture * tex)
{
	this->texture = tex;
}

RULES_Texture * Model::getTexture()
{
	return this->texture;
}

BoundingBox * Model::getBoundingBox()
{
	return &this->boundingBox;
}
