// MotionBlur.cpp: implementation of the MotionBlur class.
//
//////////////////////////////////////////////////////////////////////

#include "MotionBlur.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

MotionBlur::MotionBlur()
{
	this->createRenderTexture(1024,3,GL_RGB);
	this->factor = 1;
	this->multiplicateur = -1;
}

MotionBlur::~MotionBlur()
{
	glDeleteTextures(1,&textureID);
}

void MotionBlur::createRenderTexture(int size, int channels, int type)										
{
	//le pointeur sur la texture vide envoyer  la carte
	unsigned int *pTexture = NULL;											

	//alloue et initialise la mmoire
	pTexture = new unsigned int [size * size * channels];
	memset(pTexture, 0, size * size * channels * sizeof(unsigned int));	

	//Alloue un objet de texture OpenGL
	glGenTextures(1, &textureID);								
	glBindTexture(GL_TEXTURE_2D, textureID);					
	
	//envoie les donne de la texture avec ces paramtre
	glTexImage2D(GL_TEXTURE_2D, 0, channels, size, size, 0, type, GL_UNSIGNED_INT, pTexture);
	// Set the texture quality
	glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MAX_ANISOTROPY_EXT, 2.0f); 
	glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_LINEAR_MIPMAP_LINEAR); 
	glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_LINEAR); 

	// release de la mmoire
	delete [] pTexture;																					
}

void MotionBlur::calcFactor(float elapsedTime)
{
	if (factor < 1)
	{
		float value = (factor - 0.4) * (factor - 0.4);
		if (multiplicateur < 0)
		{
			value =0.2*factor + 1;
		}
		if (factor < 0.45)
		{
			factor = 0.45;
			multiplicateur = 1;
		}
		factor += value*elapsedTime * multiplicateur;
		if (factor >1)
		{
			factor = 1;
		}
	}
	else
	{
		factor = 1;
	}
}

void MotionBlur::draw()
{
	if (factor < 1)
	{
		float tempFactor = factor;
		if (factor< 0.55)
		{
			tempFactor = 0.55;
		}
		glViewport(0,0,1024,768);
		glMatrixMode(GL_PROJECTION);
		glLoadIdentity();
		gluPerspective(110 - 20*tempFactor,1024/768,1,10000);
		glMatrixMode(GL_MODELVIEW);
		// on desactive le depth test
		glDisable(GL_DEPTH_TEST);
		// on passe en mode projection
		glMatrixMode(GL_PROJECTION);
			
					// on sauvegarde la matrice de projection
		glPushMatrix();
		
			// on nettoie la matrice de projection
			glLoadIdentity();
			
			// on passe en mode ortho de la taille de l'ecran
			glOrtho(0,1024,0,768,-1,1);	
			
			// on passe en mode modelview
			glMatrixMode(GL_MODELVIEW);
			
			// on sauvegarde la matrice de modelview
			glPushMatrix();
			
				// on nettoie la matrice de projection
				glLoadIdentity();
				
				glEnable(GL_BLEND);
				glBlendFunc(GL_ONE_MINUS_SRC_ALPHA, GL_SRC_ALPHA);
				glBindTexture(GL_TEXTURE_2D,textureID);
				glTexEnvf(GL_TEXTURE_ENV,GL_TEXTURE_ENV_MODE,GL_MODULATE);
				glColor4f(1,1,0.5,tempFactor);
				float force = 8;
				glBegin(GL_QUADS);
					glTexCoord2f(0,0);
					glVertex2f(0-force,0-force);
					glTexCoord2f(1,0);
					glVertex2f(1024+force,0-force);
					glTexCoord2f(1,1);
					glVertex2f(1024+force,1024+force);
					glTexCoord2f(0,1);
					glVertex2f(0-force,1024+force);
				glEnd();

				// on selectionne la matrice de projection
				glMatrixMode(GL_PROJECTION);
			
			// et on recupere la matrice de projection sauvegard precedement
			glPopMatrix();	
			
			// on selectionne la matrice de projection modelview
			glMatrixMode(GL_MODELVIEW);
		glPopMatrix();
		// on desactive le depth test
		glEnable(GL_DEPTH_TEST);
	
	glDisable(GL_BLEND);

	// on se bind sur la texture de motionblur
	glBindTexture(GL_TEXTURE_2D, textureID);
	// on recupere la texture
	glCopyTexImage2D(GL_TEXTURE_2D, 0, GL_RGB, 0, 0, 1024, 1024, 0);

	}
}

void MotionBlur::start()
{
	/*	// on se bind sur la texture de motionblur
		glBindTexture(GL_TEXTURE_2D, textureID);
		// on recupere la texture
		glCopyTexImage2D(GL_TEXTURE_2D, 0, GL_RGB, 0, 0, 1024, 1024, 0);
*/
		this->factor -= 0.001;
		this->multiplicateur = -1;
}

void MotionBlur::stop()
{
		this->factor =1;
		this->multiplicateur = 1;
}
