// ParticleEngine.h: interface for the ParticleEngine class.
//
//////////////////////////////////////////////////////////////////////

#ifndef PARTICLEENGINE_H
#define PARTICLEENGINE_H

#ifdef WIN32
#include <windows.h>
#endif
#include <GL/gl.h>
#include "RULES_Vector.h"
#include "RULES_Texture.h"
#include <GL/glut.h>
#include <math.h>
#include <stdlib.h>

/**
* La classe permettant de grer le rendu et le calcul physique des particules.
*
* La classe ParticleEngine implmente le pattern SINGLETON.
*/
class ParticleEngine  
{
private:

	/**
	* La structure reprsentant une particule simplifi
	*
	* Comment par Hillaire Sbastien
	*/
	struct Particle
	{
		/**
		* la position de la particule dans l'espace homogne
		*/
		RULES_Vector position;
		/**
		* la vie restante de la particule
		*/
		float vie;
		/**
		* La direction dans laquel se dirige la particule
		*/
		RULES_Vector direction;
		/**
		* La couleur de la particule
		*/
		float color[3];
		/**
		* vaut true si on doit rinitialis la particule
		*/
		bool init;
		/**
		* l'angle de rotation de la particule
		*/
		float rotation;
	};

	/**
	* Le nombre de particule
	*/
	int particleNumber;
	/**
	* Le pointeur sur le tableau des particules
	*/
	Particle * particles;

	/**
	* Le temps depuis le dbut de l'application enregistrer  la dernire frame
	*/
	float previousTime;
	/**
	* Le temps depuis le dbut de l'application enregistrer  la dernire frame
	*/
	float beginTime;
	/**
	* Le temps coul depuis l'image prcedente
	*/
	float delay;

	/**
	* La position prcedente du gnerateur de particule
	*/
	RULES_Vector previousPosition;

	/**
	* L'indice de vrification de regneration de particule(incrmenter  chaque frame)
	*/
	int currentIndice;

	/**
	* La texture  appliquer au particule
	*/
	RULES_Texture * image;

	/**
	* Constructeur priv
	*/
	ParticleEngine();

public:
	/**
	* Destructeur public
	*/
	virtual ~ParticleEngine();
	/**
	* Permet d'obtenir l'unique instance du moteur de marticule
	*
	* @return le pointeur sur l'unique instance du moteur de marticule
	*/
	inline static ParticleEngine * getInstance(){static ParticleEngine part;return &part;};

	/**
	* Affecte le nombre de particule du moteur.
	* REM : peut tre changer  tout moment mais trs consommateur de ressource(rallocation mmoire)
	*/
	void setParticlesNumber( int number);

	/**
	* Affecte la position prsdente du gnerateur.
	* Utilis pour le type de gneration de particule de la souris) :
	*	- la souris bouge : on init les particules  la position de la souris
	*	- la souris ne bouge pas : on lance les particule en l'air
	*/
	void setPreviousPosition(RULES_Vector * pos);

	/**
	* Permet au moteur de calculer le prochaine tat de tout les particules et de les rgnerer.
	* Ensuite, les particules sont affiches.=> a appel qu'une seul fois par frame.
	*/
	void frame(RULES_Vector * pos);

};

#endif
