// PlayList.cpp: implementation of the PlayList class.
//
//////////////////////////////////////////////////////////////////////

#include "PlayList.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

PlayList::PlayList()
{
	// par defaut, la playlist n'est pas en mode random
	random = false;
	// l'indice du MP3 courant est mis a un indice invalide
	indice = -1;
	// et le repertoire de base est mis a un repertoire quelconque
	baseRep = "z:/mp3/";
	// la playliste n'est pas valide...
	ok = false;
}

PlayList::~PlayList()
{
	// on delete chaque MP3 de la playList
	int size = mp3.size();
	for (int i = 0; i < size; i++)
	{
		mp3[i].clear();
		mp3[i].resize(0);
	}
	// on nettoie le vecteur
	mp3.clear();
	// et on lui donne une taille de 0
	mp3.resize(0);
}

void PlayList::readPlayList(char *file)
{
	std::ifstream fichier;

	FILE *f;

	f=fopen(file,"r");
	if(f!=NULL)
	{
		// on ouvre le fichier
		fichier.open(file);
	}

	// on vrifie que l'ouverture s'est bien passe
	if(f!=NULL)
	{
		while (!fichier.eof())
		{
			// boucle qui lit dans le fichier jusqu'a ce qu'on trouve
			// le caractere \n qui indique la fin de la ligne
			// il faudrait regarder la doc de ifstream car il y a surement une methode qui le fait pour nous...
			char tmp_char = fichier.get();
			// l'utilisation d'un vecteur pour lire une ligne n'est pas judicieuse, il faudrait mieux utiliser un tableau
			// avec une taille donne, mais cela permet de ne pas avoire a se soucier des probleme de depassements...
			// (on aurait aussi pu utiliser des std::string ou des String autre...
			std::vector <char> ligne;
			
			// on ajoute le repertoire de base
			int baseSize = strlen(baseRep);
			for (int i = 0; i < baseSize; i++)
			{
				ligne.push_back(baseRep[i]);
			}

			// permet de savoir si la ligne est un commentaire
			bool trait = ((tmp_char != '#')&&(!fichier.eof()));
			while ((tmp_char!='\n')&&(!fichier.eof()))
			{
				if (tmp_char == '\\')
				{
					tmp_char = '/';
				}
				// le caractere l'est pas le retour chariot dont on l'ajoute au vecteur
				ligne.push_back(tmp_char);
				
				// on lit le caractere suivant
				tmp_char = fichier.get();
			}

			// on verifie si la ligne est un commentaire
			if (trait)
			{
				mp3.push_back(ligne);
			}

			// puis on nettois le vecteur
			ligne.clear();
			ligne.resize(0);

		}
		fichier.close();
		ok = true;// la lecture c'est bien passe

	}
	else
	{
		// impossible d'ouvrire le fichier
		fichier.close();
		// la playlist n'est pas valide
		ok = false;
	}

}

char * PlayList::getNextMp3()
{
	if (random)
	{
		// en mode aleatoire on choisit un index de MP3 au hazard
		indice = fabs(cos((float)rand()))*mp3.size();
	}
	else
	{
		// sinon on choisi le MP3 suivant
		if (indice < mp3.size())
		{
			indice ++;
		}
		else
		{
			indice = 0;
		}
	}

	// on copie le MP3 pour le renvoyer
	int sizeMp3 = mp3[indice].size();
	char * ret = new char[sizeMp3+1];
	// TODO utiliser memcpy 
	for (int i = 0; i < sizeMp3; i++)
	{
		ret[i] = mp3[indice][i];
	}
	ret[sizeMp3] = '\0';
	return ret;
}

void PlayList::setRandom(bool rand)
{
	this->random = rand;
}

bool PlayList::isRandom()
{
	return this->random;
}

bool PlayList::isOk()
{
	return ok;
}

void PlayList::readBaseRep(char *file)
{
	// lecture du fichier de base de la playlist
	std::ifstream fichier;

	FILE *f;

	f=fopen(file,"r");
	if(f!=NULL)
	{
		// on ouvre le fichier
		fichier.open(file);
	}

	// on vrifie que l'ouverture s'est bien passe
	if(f!=NULL)
	{
		std::vector <char> ligne;
		while (!fichier.eof())
		{
			// ici aussi, l'utilisation d'une methode approprie et d'une std::string
			// peut etre interessante.
			char tmp_char = fichier.get();
			while ((tmp_char != '\n') && (!fichier.eof()))
			{
				if (tmp_char == '\\')
				{
					tmp_char = '/';
				}
				// le caractere l'est pas le retour chariot dont on l'ajoute au vecteur
				ligne.push_back(tmp_char);
				
				// on lit le caractere suivant
				tmp_char = fichier.get();
			}
			int size = ligne.size();
			if (size !=0)
			{
				baseRep = new char[size + 2];
				for (int i = 0; i < size; i++)
				{
					baseRep[i] = ligne[i];
				}
				baseRep[size] = '/';
				baseRep[size+1] = '\0';
			}
			ligne.clear();
		}
	}
}
