// PlayList.h: interface for the PlayList class.
//
//////////////////////////////////////////////////////////////////////

/**
 * documentation ralis par Michel de VERDELHAN : michel.deverdelhan@free.fr
 * le 20/10/2004
 */

#ifndef PLAYLIST
#define PLAYLIST_H

#include <vector>
#include <fstream>
#include <math.h>
#include <stdio.h>

/**
 * Classe gerant une playliste winamp de base.
 * </BR> Une PlayListe se presente sous la forme
 * </BR> RelativePath/fichier.extention avec relative path le chemin relatif
 * par rapport a l'emplacement de la playListe.
 * </BR> malheureusement, ce chemin relatife nous obligent a stocker le chemin relatif
 * de la playListe d'origine, sinon on ne retouve pas le chemin absolue des fichier de son...
 * 
 */
class PlayList  
{
private:
	/**
	 * le vecteur contenant l'ensemble des MP3 contenu dans la playListe
	 */
	std::vector < std::vector <char> > mp3;

	/**
	 * Le chemin relatif de la playListe.
	 * </BR> ce chemin est recuperer au debut du chargement de la playListe puis est ajout au debut de
	 * chaque MP3. Ceci permet de retrouver l'emplacement absolue des fichier.
	 */
	char * baseRep;

	/**
	 * specifie si la selection des musiques se fait de facon alatoire.
	 */
	bool random;

	/**
	 * l'index du MP3 courant dans le vecteur de MP3. en cas de generation alatoire, cet index est gener aleatoirement.
	 */
	int indice;

	/**
	 * permet de savoir si la PlayListe est valide.
	 */
	bool ok;
public:
	/**
	 * methode permettant de recuperer le repertoire de base de la playListe pour la resolution des chemins absolues.
	 */
	void readBaseRep(char * file);

	/**
	 * methode permettant de savoire si la playListe est valide
	 * @return true: la playListe est valide
	 */
	bool isOk();

	/**
	 * methode permettant de savoire si la playListe est ger aleatoirement
	 * @return true: la playListe est gr alatoirement
	 */
	bool isRandom();

	/**
	 * methode permettant de definire si la playListe doit etre gr alatoirement
	 * @param rand : true => gestion alatoire
	 */
	void setRandom(bool rand);

	/**
	 * methode qui retourne le prochain MP3 de la playListe selon la methode de gestion utilise (rand ou pas)
	 * @return char*: le nom du prochain MP3 en chemin absolue.
	 */
	char * getNextMp3();

	/**
	 * methode qui lit un fichier de playListe et stock les information dans le vecteur de MP3 de la classe
	 * @param file: le nom du fichier a ouvrir. <B>Attention</B>, le char* retourn est une copie que l'utilisateur doit 
	 * supprimer en utilisant delete (memoire allou en utilisant new)
	 */
	void readPlayList(char * file);

	/**
	 * le constructeur par defaut, cre une playListe vide non valide, pour rendre la playListe valide il faut 
	 * charger une playListe winamp
	 */
	PlayList();

	/**
	 * destructeur de la classe
	 */
	virtual ~PlayList();

};

#endif