// ollisionManager.cpp: implementation of the CollisionManager class.
//
//////////////////////////////////////////////////////////////////////

#include "collisionManager.h"
#include <iostream>

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CollisionManager::CollisionManager(Loutre *lo, Immeuble *im, Joueur *jo, int nblo, int nbim, int nbjo)
{
	loutres = lo;
	joueurs = jo;
	immeubles = im;
	loutreNumber = nblo;
	joueurNumber = nbjo;
	immeubleNumber = nbim;
}

CollisionManager::~CollisionManager()
{
	//les tableaux loutres et immeubles sont allou en dehors et ne doivent pas etre supprim
	//mme  la destruction du CollisionMananger-> autre chose s'en occupe...
}

void CollisionManager::manageCollision()
{
	int i,j;
	
	/*** dtection collision joueur-immeuble ***/
	for(j=0;j<joueurNumber;j++)
	{
		for(i=0;i<immeubleNumber;i++)
		{
			float distance = (joueurs[j].getPosition() - immeubles[i].getPosition()).length();
			
			if( distance <= (JOUEUR_RADIUS + immeubles[i].getModel()->getRadius()) )
			{
				//le  joueur est encastrer dans l'immeuble => on le dcale la ou il faut
				RULES_Vector tmp = joueurs[j].getPosition();
				tmp.normalisation();
				tmp*=(immeubles[i].getModel()->getRadius() + JOUEUR_RADIUS);
				tmp.setOneCoord(1,50);
				joueurs[j].setPos(tmp);
			}
		}
	}
	
	/*** dtection collision joueur-loutre ***/
	for(j=0;j<joueurNumber;j++)
	{
		for(i=0;i<loutreNumber;i++)
		{
			if(loutres[i].isAlive())//la loutre DOIT tre en vie
			{
				RULES_Vector tmpl;
				RULES_Vector tmpj;
				tmpl = loutres[i].getPosition();
				tmpj = joueurs[j].getPosition();
				tmpl.setOneCoord(1,0);
				tmpj.setOneCoord(1,0);
				
				float distance = (tmpl - tmpj).length();
				
				if( distance <= ((loutres[i].getModel()->getRadius() * loutres[i].getSize()) + JOUEUR_RADIUS) )
				{
					//une loutre et le joueur sont encaster
					RULES_Vector tmp = tmpl - tmpj;
					tmp.normalisation();
					
					
					if(joueurs[j].isCurrentlyMoving())
					{
						//le joueur est en train de bouger : il pousse la loutre avec force
						// dans la direction joueur/loutre
						loutres[i].setCollision_FactorPushing(2);
						loutres[i].setCollision_ForcePushing(&tmp);
					}
					else
					{
						//la loutre est stop et rebondit un peu sur le joueur pour lui pass  cot
						float tmpDot = tmp.dot3( loutres[i].getCollision_Force() );
						if(tmpDot<0)
							tmpDot=-tmpDot;
						tmp*=2*tmpDot;
						tmp+= *loutres[i].getCollision_Force();
						tmp.normalisation();

						loutres[i].setCollision_Force( &tmp );
						loutres[i].setCollision_Factor(0);
					}
						
					//dplacement pour vit inter blocage loutre
					loutres[i].moveLoutre();
					loutres[j].moveLoutre();
				}
			}
		}
	}

	/*** dtection collision loutre-loutre ***/
	for(i=0;i<loutreNumber;i++)
	{
		if(loutres[i].isAlive())
		{
			for(j=0;j<loutreNumber;j++)
			{
				if(i!=j && loutres[j].isAlive())
				{
					float distance = (loutres[i].getPosition() - loutres[j].getPosition()).length();
					
					if( distance <= ((loutres[i].getModel()->getRadius() * loutres[i].getSize()) + (loutres[i].getModel()->getRadius() * loutres[i].getSize())) )
					{
						//ici : collision entre le deux loutres
						loutres[i].setCollision_Factor(0);	//on dit qu'il y a collision pour le dplacement
						loutres[j].setCollision_Factor(0);
						//loutres[i].setCollision_FactorPushing(0);	//on annule le pushing(pour l'instant une loutre arrete le force d'une autre)
						//loutres[j].setCollision_FactorPushing(0);

						float tmpcoef;
						RULES_Vector tmp;

						//on echange leur caractristique pour simul un rebondissement

						//echange de la force
						tmp = *loutres[i].getCollision_Force();
						loutres[i].setCollision_Force(loutres[j].getCollision_Force());
						loutres[j].setCollision_Force(&tmp);

						//change du coefficient force
						tmpcoef = loutres[i].getCollision_Factor();
						loutres[i].setCollision_Factor( loutres[j].getCollision_Factor() );
						loutres[j].setCollision_Factor( tmpcoef );
						
						//echange de la forcepushing
						tmp = *loutres[i].getCollision_ForcePushing();
						loutres[i].setCollision_ForcePushing(loutres[j].getCollision_ForcePushing());
						loutres[j].setCollision_ForcePushing(&tmp);

						//change du coefficient forcePushing
						tmpcoef = loutres[i].getCollision_FactorPushing();
						loutres[i].setCollision_FactorPushing( loutres[j].getCollision_FactorPushing() );
						loutres[j].setCollision_FactorPushing( tmpcoef );
						
						//Corrige le bug des loutres volantes!!!!
						loutres[i].getCollision_Force()->setOneCoord(1,0);
						//Corrige le bug des loutres volantes!!!!
						loutres[j].getCollision_Force()->setOneCoord(1,0);
						
						//dplacement pour vit blocage loutre
						loutres[i].moveLoutre();
						loutres[j].moveLoutre();
					}
				}
			}
		}
	}

}

void CollisionManager::setLoutreNumber(int nbl)
{
	loutreNumber = nbl;
}

void CollisionManager::setImmeubleNumber(int nbi)
{
	immeubleNumber = nbi;
}

void CollisionManager::setJoueurNumber(int nbj)
{
	joueurNumber = nbj;
}

void CollisionManager::setJoueurs(Joueur* j)
{
	joueurs = j;
}

void CollisionManager::setLoutres(Loutre* l)
{
	loutres = l;
}

void CollisionManager::setImmeubles(Immeuble* i)
{
	immeubles = i;
}
